/*
 *  arch/i386/snsc_mpu300/boot-time.c
 *
 *  this is based on arch/mips/ubx/boot-time.c
 *
 *  Copyright 2002,2003 Sony Corporation
 *
 *  This program is free software; you can redistribute	 it and/or modify it
 *  under  the terms of	 the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the	License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS PROVIDED	  ``AS	IS'' AND   ANY	EXPRESS OR IMPLIED
 *  WARRANTIES,	  INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO	EVENT  SHALL   THE AUTHOR  BE	 LIABLE FOR ANY	  DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED	  TO, PROCUREMENT OF  SUBSTITUTE GOODS	OR SERVICES; LOSS OF
 *  USE, DATA,	OR PROFITS; OR	BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN	 CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/config.h>

#include <linux/kernel.h>
#include <linux/proc_fs.h>
#include <linux/module.h>
#include <linux/seq_file.h>

#include <asm/io.h>
#include <asm/msr.h>
#include <asm/string.h>
#include <asm/uaccess.h>

#include <asm/snsc_mpu300.h>
#include <linux/snsc_boot_time.h>
#ifdef CONFIG_SNSC_BOOT_TIME_USE_NBLARGS
#include <linux/snsc_nblargs.h>
#endif

static void
mach_count_set(struct boot_time_entry *pEntry)
{
	rdtsc(pEntry->count_lo, pEntry->count_hi);
}

static void
mach_print(struct seq_file *m, struct boot_time_entry *pEntry)
{
	u32 ms, rm;

	/* divide 64bit value (count_hi:count_lo) / cpu_khz */
	__asm__ __volatile__("divl %2"
			     : "=a" (ms), "=d" (rm)
			     : "r" (cpu_khz), "0" (pEntry->count_lo),
			     "1" (pEntry->count_hi));
	seq_printf(m, " : 0x%08x 0x%08x : %u\n", 
		   pEntry->count_hi, pEntry->count_lo, ms);
}

static struct boot_time_md btmd = {
	base:      (void *)(MPU300_BOOT_TIME_BASE + PAGE_OFFSET),
	init:      NULL,
	count_set: mach_count_set,
	print:     mach_print,
};

static inline void
reserve_default_boottime_region()
{
	reserve_bootmem(MPU300_BOOT_TIME_BASE, MPU300_BOOT_TIME_SIZE);
	printk(KERN_INFO "reserved memory at 0x%08x size 0x%x\n",
	       MPU300_BOOT_TIME_BASE, MPU300_BOOT_TIME_SIZE);
}

void
mach_boot_time_init()
{
	static int initialized = 0;
#ifdef CONFIG_SNSC_BOOT_TIME_USE_NBLARGS
	struct nblargs_entry na;
#endif

	if (initialized)
		return;

	initialized = 1;

#ifdef CONFIG_SNSC_BOOT_TIME_USE_NBLARGS
	if (nblargs_get_key(SNSC_BOOT_TIME_KEY, &na) < 0) {
		printk(KERN_INFO "NBLArgs key \"" SNSC_BOOT_TIME_KEY
		       "\" not found, using default(0x%08x)\n",
		       btmd.base - PAGE_OFFSET);
		reserve_default_boottime_region();
	} else {
		printk(KERN_INFO "NBLArgs key \"" SNSC_BOOT_TIME_KEY
		       "\" found(0x%08x)", na.addr);
		if (na.size < ENTRY_DATA_SIZE) {
			printk(", but size(0x%x) is too small\n", na.size);
			return;
		}
		printk("\n");
		btmd.base = (void *)na.addr + PAGE_OFFSET;
		/* specified boottime region is already reserved by
		   nblargs_init() */
	}
#else
	reserve_default_boottime_region();
#endif
	boot_time_init(&btmd);
}
