/*
 *  linux/arch/i386/snsc_mpu300/init.S
 *
 *  startup code for Sony NSC MPU-300 board
 *
 *  Copyright 2001-2003 Sony Corporation.
 *
 *  Descriptor table is copied from linux/arch/i386/boot/setup.S
 *  Copyright (C) 1991, 1992 Linus Torvalds		
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/config.h>
#include <linux/threads.h>
#include <linux/linkage.h>
#include <asm/segment.h>
#include <asm/page.h>
#include <asm/pgtable.h>
#include <asm/desc.h>
#include <asm/e820.h>

/*
 *  Boot parameters:
 *    %esi is used as a pointer to the real-mode data(i.e. boot parameters)
 *    in i386 startup code(arch/i386/kernel/head.S). 
 *    So bootloader makes boot parameters in appropriate address and 
 *    this code sets %esi to that address.
 */
#define PARAM               (0x10000)
#define SCREEN_INFO         (PARAM + 0)
#define EXT_MEM_K           (PARAM + 2)
#define ALT_MEM_K           (PARAM + 0x1e0)
#define E820_MAP_NR         (PARAM + E820NR)
#define E820_MAP            (PARAM + E820MAP)
#define APM_BIOS_INFO       (PARAM + 0x40)
#define DRIVE_INFO          (PARAM + 0x80)
#define SYS_DESC_TABLE      (PARAM + 0xa0)
#define MOUNT_ROOT_RDONLY   (PARAM + 0x1F2)
#define RAMDISK_FLAGS       (PARAM + 0x1F8)
#define ORIG_ROOT_DEV       (PARAM + 0x1FC)
#define AUX_DEVICE_INFO     (PARAM + 0x1FF)
#define LOADER_TYPE         (PARAM + 0x210)
#define KERNEL_START        (PARAM + 0x214)
#define INITRD_START        (PARAM + 0x218)
#define INITRD_SIZE         (PARAM + 0x21c)
#define CL_POINTER          (PARAM + 0x228)

#define PM_EVENTS           (PARAM + 0x180)
	
/*
 *  Bootloader parameters:
 *    parameters passed by bootloader exec command
 *      * version is (BL_MAGIC) - 0x734e7343
 */
#define BLPARAM             (0x90000)
#define BL_MEMSIZE          (BLPARAM + 0x00)
#define BL_CMDLINE          (BLPARAM + 0x04)
#define BL_MAGIC            (BLPARAM + 0x08)
#define BL_INITRD_START     (BLPARAM + 0x0c)
#define BL_INITRD_SIZE      (BLPARAM + 0x10)
#define BL_E820_BASE        (BLPARAM + 0x14)   /* only version >= 1 */
#define BL_E820_NR          (BLPARAM + 0x18)   /* only version >= 1 */
#define BL_PM_EVENTS        (BLPARAM + 0x1c)   /* only version >= 2 */

#define LOADER_MAGIC  0x734e7343

/*
 *  snsc_mpu300_init
 *
 *    startup code for Sony NSC MPU-300 board
 *    this code substitutes for arch/i386/boot/head.S
 */
ENTRY(snsc_mpu300_init)

/*
 * make bootup parameters 
 */

	/* clear parameter region */
	xorl	%eax, %eax
	movl	$(PARAM), %edi
	mov	$0x90, %ecx
	cld
	rep
	stosl

	/* 0xffffffff means that PM_EVENTS is not supported */
	movl	$0xffffffff, PM_EVENTS

	movl	$(BL_MAGIC), %edi
	movl	(%edi), %eax
	cmpl	$(LOADER_MAGIC + 1), %eax
	jb	_no_e820

	cmpl	$(LOADER_MAGIC + 2), %eax
	jb	_e820

	/*
	 * Power management status (supported when version >= 2)
	 */
	movl	$(BL_PM_EVENTS), %edi
	movl	(%edi), %eax
	movl	%eax, PM_EVENTS
	
_e820:
	/*
	 * Memory parameters (e820 is supported when version >= 1)
	 */
	movl	$(BL_E820_NR), %edi
	movl	(%edi), %eax
	movb	%al, E820_MAP_NR
	lea	(%eax, %eax, 4), %ecx   /* MAP_NR x (20/4) -> MAP_NR x 5 */
	movl	$(BL_E820_BASE), %edi
	movl	(%edi), %esi
	movl	$(E820_MAP), %edi
	rep
	movsl
	jmp	_e820_end

_no_e820:
	movb	$0, E820_MAP_NR

_e820_end:
	movw	$0, EXT_MEM_K
	movl	$(BL_MEMSIZE), %edi
	movl	(%edi), %eax
	shll	$10, %eax		# convert to Kbyte
	movl	%eax, ALT_MEM_K

	/* HDD
	   XXX: now only clear drive_info region */

	/* no MCA */
	movw	$0, SYS_DESC_TABLE

	/* PS mouse
	   XXX: PS/2 mouse is not probed in boot loader */
	movw	$0, AUX_DEVICE_INFO

	/* Command line */
	movl	$(BL_CMDLINE), %edi
	movl	(%edi), %eax
	movl	%eax, CL_POINTER

	/* Check Magic number */
	movl	$(BL_MAGIC), %edi
	movl	(%edi), %eax
	cmpl	$(LOADER_MAGIC), %eax
	jne	1f

	/* INITRD */
	movl	$(BL_INITRD_START), %edi
	movl	(%edi), %eax
	movl	%eax, INITRD_START
	movl	$(BL_INITRD_SIZE), %edi
	movl	(%edi), %eax
	movl	%eax, INITRD_SIZE
	movl	$1, %eax
	movb	%al, LOADER_TYPE

1:
	/* finally, set top address of bootup parameters to %esi */
	movl	$(PARAM), %esi
	
/* 
 * initialize GDT so that the segment structure is suitable for linux kernel
 * i.e. make the same descriptor table as boot/setup.S
 */
	lgdt	mpu300_gdt_48-__PAGE_OFFSET
	ljmp	$(__KERNEL_CS), $_snsc_mpu300_ret-__PAGE_OFFSET
	
/*
 * Descriptor tables from boot/setup.S
 */
mpu300_gdt:
        .word   0, 0, 0, 0                             # dummy
        .word   0, 0, 0, 0                             # unused
					              
        .word   0xFFFF                                 # 4Gb - (0x100000*0x1000 = 4Gb)
        .word   0                                      # base address = 0
        .word   0x9A00                                 # code read/exec
        .word   0x00CF                                 # granularity = 4096, 386
                                                       #  (+5th nibble of limit)
					              
        .word   0xFFFF                                 # 4Gb - (0x100000*0x1000 = 4Gb)
        .word   0                                      # base address = 0
        .word   0x9200                                 # data read/write
        .word   0x00CF                                 # granularity = 4096, 386
                                                       #  (+5th nibble of limit)
mpu300_gdt_48:				              
        .word   0x8000                                 # gdt limit=2048,

        .long SYMBOL_NAME(mpu300_gdt)-__PAGE_OFFSET    # gdt base
