/*
 *  linux/arch/mips/ite-boards/snsc_mpu210/halt.c
 *
 *  Restart/halt/shutdown routine for Sony NSC MPU-210
 *
 *  Copyright 2003 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/kernel.h>
#include <linux/delay.h>
#include <linux/snsc_it8172_gpio.h>
#include <asm/it8172/it8172.h>
#include <asm/it8172/snsc_mpu210.h>

/* Self reset function on IT8172G GPIO16 */
#define SELF_RESET_PORT    IT8172_GPIO_PORT_C
#define SELF_RESET_BIT     0
#define SELF_RESET_NAME    "self-reset"

/* Software power control function on ITE8172G GPIO17 */
#define POWER_ON_PORT      IT8172_GPIO_PORT_C
#define POWER_ON_BIT       1
#define POWER_ON_NAME      "power-on"

void it8172_restart(char *command);
void it8172_halt(void);


void snsc_mpu210_restart(char *command)
{
	/* Try self reset function */
	/* Set GPIO pin to H */
	it8172_gpio_register_bit(SELF_RESET_PORT, SELF_RESET_BIT,
	                         SELF_RESET_NAME);
	it8172_gpio_set_mode_bit(SELF_RESET_PORT, SELF_RESET_BIT,
	                         IT8172_GPIO_MODE_OUT_HIGH);

	/* Wait 1us */
	udelay(1);

	/* Set GPIO pin to L */
	it8172_gpio_set_data_bit(SELF_RESET_PORT, SELF_RESET_BIT, 0);
	/* Set GPIO to input mode */
	it8172_gpio_set_mode_bit(SELF_RESET_PORT, SELF_RESET_BIT,
	                         IT8172_GPIO_MODE_OUT_HIGH);
	it8172_gpio_unregister_bit(SELF_RESET_PORT, SELF_RESET_BIT);

	/* Self reset function is disabled */
//	printk("snsc_mpu210_restart: Self reset function is disabled\n");
	/* Call generic restart routine */
	it8172_restart(command);
}

void snsc_mpu210_halt(void)
{
	int	i;

	/* Try software power off function */
	/* Set GPIO pin to H */
	it8172_gpio_register_bit(POWER_ON_PORT, POWER_ON_BIT, POWER_ON_NAME);
	it8172_gpio_set_mode_bit(POWER_ON_PORT, POWER_ON_BIT,
	                         IT8172_GPIO_MODE_OUT_HIGH);

	/* Wait 1s */
	for (i = 0; i < 1024; i++)
		udelay(1024);

	/* Set GPIO pin to L */
	it8172_gpio_set_data_bit(POWER_ON_PORT, POWER_ON_BIT, 0);
	/* Set GPIO to input mode */
	it8172_gpio_set_mode_bit(POWER_ON_PORT, POWER_ON_BIT,
	                         IT8172_GPIO_MODE_OUT_HIGH);
	it8172_gpio_unregister_bit(POWER_ON_PORT, POWER_ON_BIT);

	/* Software power off function is disabled */
	/* Call generic halt routine */
	it8172_halt();
}
