/*
 * linux/arch/mips/tx4925/toshiba_rbtx4925/irq.c
 *
 * Toshiba RBTX4925 specific interrupt handlers
 *
 * Author: MontaVista Software, Inc.
 *         source@mvista.com
 *
 * Copyright 2001-2002 MontaVista Software Inc.
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 *  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 *  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
IRQ  Device  

16   TX4925-CP0/00 Software 0
17   TX4925-CP0/01 Software 1
18   TX4925-CP0/02 Cascade TX4925-CP0
19   TX4925-CP0/03 Multiplexed -- do not use 
20   TX4925-CP0/04 Multiplexed -- do not use 
21   TX4925-CP0/05 Multiplexed -- do not use 
22   TX4925-CP0/06 Multiplexed -- do not use 
23   TX4925-CP0/07 CPU TIMER

24   TX4925-PIC/00 
25   TX4925-PIC/01
26   TX4925-PIC/02 Cascade RBTX4925-IOC
27   TX4925-PIC/03 RBTX4925 RTL-8019AS Ethernet
28   TX4925-PIC/04
29   TX4925-PIC/05
30   TX4925-PIC/06
31   TX4925-PIC/07
32   TX4925-PIC/08
33   TX4925-PIC/09
34   TX4925-PIC/10
35   TX4925-PIC/11
36   TX4925-PIC/12 TX4925 SerialIO Channel 0
37   TX4925-PIC/13 TX4925 SerialIO Channel 1
38   TX4925-PIC/14
39   TX4925-PIC/15
40   TX4925-PIC/16
41   TX4925-PIC/17
42   TX4925-PIC/18
43   TX4925-PIC/19
44   TX4925-PIC/20
45   TX4925-PIC/21
46   TX4925-PIC/22
47   TX4925-PIC/23
48   TX4925-PIC/24
49   TX4925-PIC/25
50   TX4925-PIC/26
51   TX4925-PIC/27
52   TX4925-PIC/28
53   TX4925-PIC/29
54   TX4925-PIC/30
55   TX4925-PIC/31

56 RBTX4925-IOC/00 PCI-D
57 RBTX4925-IOC/01 PCI-C
58 RBTX4925-IOC/02 PCI-B
59 RBTX4925-IOC/03 PCI-A
60 RBTX4925-IOC/04
61 RBTX4925-IOC/05
62 RBTX4925-IOC/06
63 RBTX4925-IOC/07
*/

#include <linux/config.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/mm.h>
#include <linux/swap.h>
#include <linux/ioport.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/pci.h>
#include <linux/timex.h>
#include <asm/bootinfo.h>
#include <asm/page.h>
#include <asm/bootinfo.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/pci.h>
#include <asm/processor.h>
#include <asm/ptrace.h>
#include <asm/reboot.h>
#include <asm/time.h>
#include <linux/version.h>
#include <linux/bootmem.h>
#include <linux/blk.h>
#include <linux/tx4925_sio.h>
#include <asm/tx4925/toshiba_rbtx4925.h>

#undef TOSHIBA_RBTX4925_IRQ_DEBUG

#ifdef TOSHIBA_RBTX4925_IRQ_DEBUG
#define TOSHIBA_RBTX4925_IRQ_NONE        0x00000000

#define TOSHIBA_RBTX4925_IRQ_INFO          ( 1 <<  0 )
#define TOSHIBA_RBTX4925_IRQ_WARN          ( 1 <<  1 )
#define TOSHIBA_RBTX4925_IRQ_EROR          ( 1 <<  2 )

#define TOSHIBA_RBTX4925_IRQ_IOC_INIT      ( 1 << 10 )
#define TOSHIBA_RBTX4925_IRQ_IOC_STARTUP   ( 1 << 11 )
#define TOSHIBA_RBTX4925_IRQ_IOC_SHUTDOWN  ( 1 << 12 )
#define TOSHIBA_RBTX4925_IRQ_IOC_ENABLE    ( 1 << 13 )
#define TOSHIBA_RBTX4925_IRQ_IOC_DISABLE   ( 1 << 14 )
#define TOSHIBA_RBTX4925_IRQ_IOC_MASK      ( 1 << 15 )
#define TOSHIBA_RBTX4925_IRQ_IOC_ENDIRQ    ( 1 << 16 )

#define TOSHIBA_RBTX4925_IRQ_ISA_INIT      ( 1 << 20 )
#define TOSHIBA_RBTX4925_IRQ_ISA_STARTUP   ( 1 << 21 )
#define TOSHIBA_RBTX4925_IRQ_ISA_SHUTDOWN  ( 1 << 22 )
#define TOSHIBA_RBTX4925_IRQ_ISA_ENABLE    ( 1 << 23 )
#define TOSHIBA_RBTX4925_IRQ_ISA_DISABLE   ( 1 << 24 )
#define TOSHIBA_RBTX4925_IRQ_ISA_MASK      ( 1 << 25 )
#define TOSHIBA_RBTX4925_IRQ_ISA_ENDIRQ    ( 1 << 26 )

#define TOSHIBA_RBTX4925_SETUP_ALL         0xffffffff
#endif

#ifdef TOSHIBA_RBTX4925_IRQ_DEBUG
static const u32 toshiba_rbtx4925_irq_debug_flag = (TOSHIBA_RBTX4925_IRQ_NONE
						    | TOSHIBA_RBTX4925_IRQ_INFO
						    | TOSHIBA_RBTX4925_IRQ_WARN
						    | TOSHIBA_RBTX4925_IRQ_EROR
//                                                 | TOSHIBA_RBTX4925_IRQ_IOC_INIT
//                                                 | TOSHIBA_RBTX4925_IRQ_IOC_STARTUP
//                                                 | TOSHIBA_RBTX4925_IRQ_IOC_SHUTDOWN
//                                                 | TOSHIBA_RBTX4925_IRQ_IOC_ENABLE
//                                                 | TOSHIBA_RBTX4925_IRQ_IOC_DISABLE
//                                                 | TOSHIBA_RBTX4925_IRQ_IOC_MASK
//                                                 | TOSHIBA_RBTX4925_IRQ_IOC_ENDIRQ
//                                                 | TOSHIBA_RBTX4925_IRQ_ISA_INIT
//                                                 | TOSHIBA_RBTX4925_IRQ_ISA_STARTUP
//                                                 | TOSHIBA_RBTX4925_IRQ_ISA_SHUTDOWN
//                                                 | TOSHIBA_RBTX4925_IRQ_ISA_ENABLE
//                                                 | TOSHIBA_RBTX4925_IRQ_ISA_DISABLE
//                                                 | TOSHIBA_RBTX4925_IRQ_ISA_MASK
//                                                 | TOSHIBA_RBTX4925_IRQ_ISA_ENDIRQ
    );
#endif

#ifdef TOSHIBA_RBTX4925_IRQ_DEBUG
#define TOSHIBA_RBTX4925_IRQ_DPRINTK(flag,str...) \
        if ( (toshiba_rbtx4925_irq_debug_flag) & (flag) ) \
        { \
           char tmp[100]; \
           sprintf( tmp, str ); \
           printk( "%s(%s:%u)::%s", __FUNCTION__, __FILE__, __LINE__, tmp ); \
        }
#else
#define TOSHIBA_RBTX4925_IRQ_DPRINTK(flag,str...)
#endif

#define TOSHIBA_RBTX4925_IRQ_ISA_BEG MI8259_IRQ_ISA_BEG
#define TOSHIBA_RBTX4925_IRQ_ISA_END MI8259_IRQ_ISA_END
#define TOSHIBA_RBTX4925_IRQ_ISA_MID ((TOSHIBA_RBTX4925_IRQ_ISA_BEG+TOSHIBA_RBTX4925_IRQ_ISA_END+1)/2)

/* mvmcp -- todo these are defaults from tx4927 might not work for tx4925 */
#define TOSHIBA_RBTX4925_IRQ_NEST_IOC_ON_PIC TX4925_IRQ_NEST_EXT_ON_PIC
#define TOSHIBA_RBTX4925_IRQ_NEST_ISA_ON_IOC (TOSHIBA_RBTX4925_IRQ_IOC_BEG+2)
#define TOSHIBA_RBTX4925_IRQ_NEST_ISA_ON_ISA (TOSHIBA_RBTX4925_IRQ_ISA_BEG+2)

static unsigned int toshiba_rbtx4925_irq_ioc_startup(unsigned int irq);
static void toshiba_rbtx4925_irq_ioc_shutdown(unsigned int irq);
static void toshiba_rbtx4925_irq_ioc_enable(unsigned int irq);
static void toshiba_rbtx4925_irq_ioc_disable(unsigned int irq);
static void toshiba_rbtx4925_irq_ioc_mask_and_ack(unsigned int irq);
static void toshiba_rbtx4925_irq_ioc_end(unsigned int irq);

static spinlock_t toshiba_rbtx4925_ioc_lock = SPIN_LOCK_UNLOCKED;

#define TOSHIBA_RBTX4925_IOC_NAME "RBTX4925-IOC"
static struct hw_interrupt_type toshiba_rbtx4925_irq_ioc_type = {
	typename:TOSHIBA_RBTX4925_IOC_NAME,
	startup:toshiba_rbtx4925_irq_ioc_startup,
	shutdown:toshiba_rbtx4925_irq_ioc_shutdown,
	enable:toshiba_rbtx4925_irq_ioc_enable,
	disable:toshiba_rbtx4925_irq_ioc_disable,
	ack:toshiba_rbtx4925_irq_ioc_mask_and_ack,
	end:toshiba_rbtx4925_irq_ioc_end,
	set_affinity:NULL
};
#define TOSHIBA_RBTX4925_IOC_INTR_ENAB 0xbb002000
#define TOSHIBA_RBTX4925_IOC_INTR_STAT 0xbb002006

u32
bit2num(u32 num)
{
	u32 i;

	for (i = 0; i < (sizeof (num) * 8); i++) {
		if (num & (1 << i)) {
			return (i);
		}
	}
	return (0);
}

int
toshiba_rbtx4925_irq_nested(int sw_irq)
{
	u32 level3;

	level3 = reg_rd08(TOSHIBA_RBTX4925_IOC_INTR_STAT) & 0xff;
	if (level3) {
		sw_irq = TOSHIBA_RBTX4925_IRQ_IOC_BEG + bit2num(level3);
		if (sw_irq != TOSHIBA_RBTX4925_IRQ_NEST_ISA_ON_IOC) {
			goto RETURN;
		}
	}

      RETURN:
	return (sw_irq);
}

#define TOSHIBA_RBTX4925_PIC_ACTION(s) { no_action, 0, 0, s, NULL, NULL }
static struct irqaction toshiba_rbtx4925_irq_ioc_action =
TOSHIBA_RBTX4925_PIC_ACTION(TOSHIBA_RBTX4925_IOC_NAME);

/**********************************************************************************/
/* Functions for ioc                                                              */
/**********************************************************************************/

void
dump_irqs(char *s)
{
#if 0
	return;
#else
	int i;
	for (i = 0; i < 64; i++) {
		printk("%s: i=%02d s=0x%02x a=0x%08x d=%1d h=0x%08x n=%s\n",
		       s, i, irq_desc[i].status, (unsigned int)irq_desc[i].action,
		       irq_desc[i].depth, (unsigned int)irq_desc[i].handler,
		       (irq_desc[i].handler ? irq_desc[i].handler->
			typename : "null"));
	}
	printk("%s: 0x%08x = 0x%02x\n", s, 0xbb002000, TX4925_RD08(0xbb002000));
	printk("%s: 0x%08x = 0x%02x\n", s, 0xbb002002, TX4925_RD08(0xbb002002));
	printk("%s: 0x%08x = 0x%02x\n", s, 0xbb002004, TX4925_RD08(0xbb002004));
	printk("%s: 0x%08x = 0x%02x\n", s, 0xbb002006, TX4925_RD08(0xbb002006));
	printk("%s: 0x%08x = 0x%02x\n", s, 0xbb003000, TX4925_RD08(0xbb003000));
	printk("%s: 0x%08x = 0x%08x\n", s, 0xff1ff610, TX4925_RD32(0xff1ff610));
	printk("%s: 0x%08x = 0x%08x\n", s, 0xff1ff614, TX4925_RD32(0xff1ff614));
	printk("%s: 0x%08x = 0x%08x\n", s, 0xff1ff618, TX4925_RD32(0xff1ff618));
	printk("%s: 0x%08x = 0x%08x\n", s, 0xff1ff61c, TX4925_RD32(0xff1ff61c));
	printk("%s: 0x%08x = 0x%08x\n", s, 0xff1ff620, TX4925_RD32(0xff1ff620));
	printk("%s: 0x%08x = 0x%08x\n", s, 0xff1ff624, TX4925_RD32(0xff1ff624));
	printk("%s: 0x%08x = 0x%08x\n", s, 0xff1ff628, TX4925_RD32(0xff1ff628));
	printk("%s: 0x%08x = 0x%08x\n", s, 0xff1ff62c, TX4925_RD32(0xff1ff62c));
	printk("%s: 0x%08x = 0x%08x\n", s, 0xff1ff640, TX4925_RD32(0xff1ff640));
	printk("%s: 0x%08x = 0x%08x\n", s, 0xff1ff680, TX4925_RD32(0xff1ff680));
	printk("%s: 0x%08x = 0x%08x\n", s, 0xff1ff6a0, TX4925_RD32(0xff1ff6a0));
#endif
}

static void __init
toshiba_rbtx4925_irq_ioc_init(void)
{
	int i;

	TOSHIBA_RBTX4925_IRQ_DPRINTK(TOSHIBA_RBTX4925_IRQ_IOC_INIT,
				     "beg=%d end=%d\n",
				     TOSHIBA_RBTX4925_IRQ_IOC_BEG,
				     TOSHIBA_RBTX4925_IRQ_IOC_END);

	for (i = TOSHIBA_RBTX4925_IRQ_IOC_BEG;
	     i <= TOSHIBA_RBTX4925_IRQ_IOC_END; i++) {
		irq_desc[i].status = IRQ_DISABLED;
		irq_desc[i].action = 0;
		irq_desc[i].depth = 3;
		irq_desc[i].handler = &toshiba_rbtx4925_irq_ioc_type;
	}

	setup_irq(TOSHIBA_RBTX4925_IRQ_NEST_IOC_ON_PIC,
		  &toshiba_rbtx4925_irq_ioc_action);

	return;
}

static unsigned int
toshiba_rbtx4925_irq_ioc_startup(unsigned int irq)
{
	TOSHIBA_RBTX4925_IRQ_DPRINTK(TOSHIBA_RBTX4925_IRQ_IOC_STARTUP,
				     "irq=%d\n", irq);

	if (irq < TOSHIBA_RBTX4925_IRQ_IOC_BEG
	    || irq > TOSHIBA_RBTX4925_IRQ_IOC_END) {
		TOSHIBA_RBTX4925_IRQ_DPRINTK(TOSHIBA_RBTX4925_IRQ_EROR,
					     "bad irq=%d\n", irq);
		panic("\n");
	}

	toshiba_rbtx4925_irq_ioc_enable(irq);

	return (0);
}

static void
toshiba_rbtx4925_irq_ioc_shutdown(unsigned int irq)
{
	TOSHIBA_RBTX4925_IRQ_DPRINTK(TOSHIBA_RBTX4925_IRQ_IOC_SHUTDOWN,
				     "irq=%d\n", irq);

	if (irq < TOSHIBA_RBTX4925_IRQ_IOC_BEG
	    || irq > TOSHIBA_RBTX4925_IRQ_IOC_END) {
		TOSHIBA_RBTX4925_IRQ_DPRINTK(TOSHIBA_RBTX4925_IRQ_EROR,
					     "bad irq=%d\n", irq);
		panic("\n");
	}

	toshiba_rbtx4925_irq_ioc_disable(irq);

	return;
}

static void
toshiba_rbtx4925_irq_ioc_enable(unsigned int irq)
{
	unsigned long flags;
	volatile unsigned char v;

	TOSHIBA_RBTX4925_IRQ_DPRINTK(TOSHIBA_RBTX4925_IRQ_IOC_ENABLE,
				     "irq=%d\n", irq);

	if (irq < TOSHIBA_RBTX4925_IRQ_IOC_BEG
	    || irq > TOSHIBA_RBTX4925_IRQ_IOC_END) {
		TOSHIBA_RBTX4925_IRQ_DPRINTK(TOSHIBA_RBTX4925_IRQ_EROR,
					     "bad irq=%d\n", irq);
		panic("\n");
	}

	spin_lock_irqsave(&toshiba_rbtx4925_ioc_lock, flags);

	v = TX4925_RD08(TOSHIBA_RBTX4925_IOC_INTR_ENAB);
	v |= (1 << (irq - TOSHIBA_RBTX4925_IRQ_IOC_BEG));
	TOSHIBA_RBTX4925_WR08(TOSHIBA_RBTX4925_IOC_INTR_ENAB, v);

	spin_unlock_irqrestore(&toshiba_rbtx4925_ioc_lock, flags);

	return;
}

static void
toshiba_rbtx4925_irq_ioc_disable(unsigned int irq)
{
	unsigned long flags;
	volatile unsigned char v;

	TOSHIBA_RBTX4925_IRQ_DPRINTK(TOSHIBA_RBTX4925_IRQ_IOC_DISABLE,
				     "irq=%d\n", irq);

	if (irq < TOSHIBA_RBTX4925_IRQ_IOC_BEG
	    || irq > TOSHIBA_RBTX4925_IRQ_IOC_END) {
		TOSHIBA_RBTX4925_IRQ_DPRINTK(TOSHIBA_RBTX4925_IRQ_EROR,
					     "bad irq=%d\n", irq);
		panic("\n");
	}

	spin_lock_irqsave(&toshiba_rbtx4925_ioc_lock, flags);

	v = TX4925_RD08(TOSHIBA_RBTX4925_IOC_INTR_ENAB);
	v &= ~(1 << (irq - TOSHIBA_RBTX4925_IRQ_IOC_BEG));
	TOSHIBA_RBTX4925_WR08(TOSHIBA_RBTX4925_IOC_INTR_ENAB, v);

	spin_unlock_irqrestore(&toshiba_rbtx4925_ioc_lock, flags);


	return;
}

static void
toshiba_rbtx4925_irq_ioc_mask_and_ack(unsigned int irq)
{
	TOSHIBA_RBTX4925_IRQ_DPRINTK(TOSHIBA_RBTX4925_IRQ_IOC_MASK, "irq=%d\n",
				     irq);

	if (irq < TOSHIBA_RBTX4925_IRQ_IOC_BEG
	    || irq > TOSHIBA_RBTX4925_IRQ_IOC_END) {
		TOSHIBA_RBTX4925_IRQ_DPRINTK(TOSHIBA_RBTX4925_IRQ_EROR,
					     "bad irq=%d\n", irq);
		panic("\n");
	}

	toshiba_rbtx4925_irq_ioc_disable(irq);

	return;
}

static void
toshiba_rbtx4925_irq_ioc_end(unsigned int irq)
{
	TOSHIBA_RBTX4925_IRQ_DPRINTK(TOSHIBA_RBTX4925_IRQ_IOC_ENDIRQ,
				     "irq=%d\n", irq);

	if (irq < TOSHIBA_RBTX4925_IRQ_IOC_BEG
	    || irq > TOSHIBA_RBTX4925_IRQ_IOC_END) {
		TOSHIBA_RBTX4925_IRQ_DPRINTK(TOSHIBA_RBTX4925_IRQ_EROR,
					     "bad irq=%d\n", irq);
		panic("\n");
	}

	if (!(irq_desc[irq].status & (IRQ_DISABLED | IRQ_INPROGRESS))) {
		toshiba_rbtx4925_irq_ioc_enable(irq);
	}

	return;
}

void __init
init_IRQ(void)
{
	extern void tx4925_irq_init(void);

	cli();

	tx4925_irq_init();
	toshiba_rbtx4925_irq_ioc_init();

#ifdef CONFIG_PCI
	{
		extern void toshiba_rbtx4925_pci_irq_init(void);
		toshiba_rbtx4925_pci_irq_init();
	}
#endif

	wbflush();

	//dump_irqs( "init_IRQ" );

	return;
}
