/*
 * linux/arch/mips/tx4925/toshiba_rbtx4925/setup.c
 *
 * Toshiba rbtx4925 specific setup
 *
 * Author: MontaVista Software, Inc.
 *         source@mvista.com
 *
 * Copyright 2001-2002 MontaVista Software Inc.
 *
 * Copyright (C) 1996, 1997, 2001  Ralf Baechle
 * Copyright (C) 2000 RidgeRun, Inc.
 * Author: RidgeRun, Inc.
 *   glonnon@ridgerun.com, skranz@ridgerun.com, stevej@ridgerun.com
 *
 * Copyright 2001 MontaVista Software Inc.
 * Author: jsun@mvista.com or jsun@junsun.net
 *
 * Copyright 2002 MontaVista Software Inc.
 * Author: Michael Pruznick, michael_pruznick@mvista.com
 *
 * Copyright (C) 2000-2001 Toshiba Corporation 
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 *  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 *  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/config.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/mm.h>
#include <linux/swap.h>
#include <linux/ioport.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/pci.h>
#include <linux/timex.h>
#include <asm/bootinfo.h>
#include <asm/page.h>
#include <asm/bootinfo.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/pci.h>
#include <asm/processor.h>
#include <asm/ptrace.h>
#include <asm/reboot.h>
#include <asm/time.h>
#include <asm/smsc_fdc37m81x.h>
#include <linux/version.h>
#include <linux/bootmem.h>
#include <linux/blk.h>
#include <linux/tx4925_sio.h>
#include <asm/tx4925/tx4925.h>
#include <asm/tx4925/tx4925_rtc.h>
#include <asm/tx4925/toshiba_rbtx4925.h>
#ifdef CONFIG_PCI
#include <asm/tx4925/tx4925_pci.h>
#include <linux/types.h>
#include <linux/pci.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <asm/pci_channel.h>
#endif
#ifdef CONFIG_PC_KEYB
#include <asm/keyboard.h>
#endif
#ifdef CONFIG_BLK_DEV_IDEPCI
#include <linux/hdreg.h>
#include <asm/ptrace.h>
#include <linux/ide.h>
extern struct ide_ops std_ide_ops;
#endif

#undef TOSHIBA_RBTX4925_SETUP_DEBUG

#ifdef TOSHIBA_RBTX4925_SETUP_DEBUG
#define TOSHIBA_RBTX4925_SETUP_NONE        0x00000000

#define TOSHIBA_RBTX4925_SETUP_INFO        ( 1 <<  0 )
#define TOSHIBA_RBTX4925_SETUP_WARN        ( 1 <<  1 )
#define TOSHIBA_RBTX4925_SETUP_EROR        ( 1 <<  2 )

#define TOSHIBA_RBTX4925_SETUP_EFWFU       ( 1 <<  3 )
#define TOSHIBA_RBTX4925_SETUP_SETUP       ( 1 <<  4 )
#define TOSHIBA_RBTX4925_SETUP_TIME_INIT   ( 1 <<  5 )
#define TOSHIBA_RBTX4925_SETUP_TIMER_SETUP ( 1 <<  6 )
#define TOSHIBA_RBTX4925_SETUP_PCIBIOS     ( 1 <<  7 )
#define TOSHIBA_RBTX4925_SETUP_PCI1        ( 1 <<  8 )
#define TOSHIBA_RBTX4925_SETUP_PCI2        ( 1 <<  9 )
#define TOSHIBA_RBTX4925_SETUP_PCI66       ( 1 << 10 )

#define TOSHIBA_RBTX4925_SETUP_ALL         0xffffffff
#endif

#ifdef TOSHIBA_RBTX4925_SETUP_DEBUG
static const u32 toshiba_rbtx4925_setup_debug_flag =
    (TOSHIBA_RBTX4925_SETUP_NONE | TOSHIBA_RBTX4925_SETUP_INFO |
     TOSHIBA_RBTX4925_SETUP_WARN | TOSHIBA_RBTX4925_SETUP_EROR |
     TOSHIBA_RBTX4925_SETUP_EFWFU | TOSHIBA_RBTX4925_SETUP_SETUP |
     TOSHIBA_RBTX4925_SETUP_TIME_INIT | TOSHIBA_RBTX4925_SETUP_TIMER_SETUP
     | TOSHIBA_RBTX4925_SETUP_PCIBIOS | TOSHIBA_RBTX4925_SETUP_PCI1 |
     TOSHIBA_RBTX4925_SETUP_PCI2 | TOSHIBA_RBTX4925_SETUP_PCI66);
#endif

#ifdef TOSHIBA_RBTX4925_SETUP_DEBUG
#define TOSHIBA_RBTX4925_SETUP_DPRINTK(flag,str...) \
        if ( (toshiba_rbtx4925_setup_debug_flag) & (flag) ) \
        { \
           char tmp[100]; \
           sprintf( tmp, str ); \
           printk( "%s(%s:%u)::%s", __FUNCTION__, __FILE__, __LINE__, tmp ); \
        }
#else
#define TOSHIBA_RBTX4925_SETUP_DPRINTK(flag,str...)
#endif

/* These functions are used for rebooting or halting the machine*/
extern void toshiba_rbtx4925_restart(char *command);
extern void toshiba_rbtx4925_halt(void);
extern void toshiba_rbtx4925_power_off(void);

extern void gt64120_time_init(void);
extern void toshiba_rbtx4925_irq_setup(void);

#ifdef CONFIG_PCI
#undef TX4925_SUPPORT_COMMAND_IO
unsigned long mips_pci_io_base; /* calculated */
unsigned long mips_pci_io_size = TX4925_PCIIO_SIZE;
unsigned long mips_pci_mem_base; /* calculated */
unsigned long mips_pci_mem_size = TX4925_PCIMEM_SIZE;
/* for legacy I/O, PCI I/O PCI Bus address must be 0 */
unsigned long mips_memory_upper;
unsigned long tx4925_memory_upper_max = 0;
static int tx4925_ccfg_toeon = 1;
static int tx4925_pcic_trdyto = -1;	/* leave default values */
unsigned long tx4925_ce_base[8];
void tx4925_pci_setup(void);
void tx4925_reset_pci_pcic(void);

void
tx4925_dump_pcic_settings(void)
{
	printk("tx4925 pcic settings:");
	{
		int i;
		unsigned long *preg = (unsigned long *) tx4925_pcicptr;
		for (i = 0; i < sizeof (struct tx4925_pcic_reg); i += 4) {
			if (i % 32 == 0)
				printk("\n%04x:", i);
			if (preg == &tx4925_pcicptr->g2pintack
			    || preg == &tx4925_pcicptr->g2pspc) {
				printk(" XXXXXXXX");
				preg++;
				continue;
			}
			printk(" %08lx", *preg++);
			if (preg == &tx4925_pcicptr->g2pcfgadrs)
				break;
		}
		printk("\n");
	}
}

static void
tx4925_pcierr_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	extern void tx4925_dump_pcic_settings(void);
	printk("PCI error interrupt (irq 0x%x).\n", irq);
	printk("pcistat:%04x, g2pstatus:%08lx, pcicstatus:%08lx\n",
	       (unsigned short) (tx4925_pcicptr->pcistatus >> 16),
	       tx4925_pcicptr->g2pstatus, tx4925_pcicptr->pcicstatus);
	printk("ccfg:%08lx\n", (unsigned long) tx4925_ccfgptr->ccfg);
	show_regs(regs);
	tx4925_dump_pcic_settings();
	panic("PCI error at PC:%08lx.", regs->cp0_epc);
}

static struct irqaction pcic_action = {
	tx4925_pcierr_interrupt, 0, 0, "PCI-C", NULL, NULL
};

static struct irqaction pcierr_action = {
	tx4925_pcierr_interrupt, 0, 0, "PCI-ERR", NULL, NULL
};

void __init
toshiba_rbtx4925_pci_irq_init(void)
{
	setup_irq(TX4925_IRQ_IRC_PCIC, &pcic_action);
	setup_irq(TX4925_IRQ_IRC_PCIERR, &pcierr_action);
	return;
}

void
tx4925_reset_pci_pcic(void)
{
	/* Reset PCIC */
	tx4925_ccfgptr->clkctr &= ~TX4925_CONFIG_CLKCTR_PCIRSTI;
	udelay(10000);
	/* clear PCIC reset */
	tx4925_ccfgptr->clkctr |= TX4925_CONFIG_CLKCTR_PCIRSTI;
}

void
print_pci_status(void)
{
	printk("PCI STATUS %lx\n", tx4925_pcicptr->pcistatus);
	printk("PCIC STATUS %lx\n", tx4925_pcicptr->pcicstatus);
}

static unsigned long __init
rbtx4925_find_free_region(int n,
                         unsigned long *bases,
                         unsigned long *sizes,
                         unsigned long begin,
                         unsigned long end,
                         unsigned long *size)
{
       unsigned long base;
       unsigned long sz = *size;
       int i;

       for (; sz; sz /= 2) {
               for (base = begin / sz * sz; base < end; base += sz) {
                       for (i = 0; i < n; i++) {
                               if (sizes[i] &&
                                          base <= bases[i] + (sizes[i] - 1) &&
                                          base + (sz - 1) >= bases[i])
                                       break;
                       }
                       if (i == n) {
                               *size = sz;
                               return base;
                       }
               }
       }
      return 0;
}                                   

static void rbtx4925_setup_pci_regions(void)
{
       unsigned long base[4+8+1];
       unsigned long size[4+8+1];
       unsigned long mbegin, mend;
       unsigned long rbtx4925_ce_base[8];
       unsigned long rbtx4925_ce_size[8];
       int i;
       
       /* SDRAMC,EBUSC are configured by PROM */
       for (i = 0; i < 8; i++) {
               if (!(tx4925_ebuscptr->ch[i].cr & 0x8))
                             continue;       /* disabled */
               rbtx4925_ce_base[i] = tx4925_ebuscptr->ch[i].bar;
               rbtx4925_ce_size[i] = 0x00100000 << ((tx4925_ebuscptr->ch[i].cr >> 8) & 0xf);
       }

       if (mips_pci_mem_base || mips_pci_io_base)
              return; /* done */

       for (i = 0; i < sizeof(base) / sizeof(base[0]); i++)
              base[i] = size[i] = 0;

       for (i = 0; i < 4; i++) {
              if (!(tx4925_sdramcptr->cr[i] & 0x00000200))
                              continue;       /* disabled */
              base[i] = tx4925_sdramcptr->cr[i] & 0xffe00000;
              size[i] = ((tx4925_sdramcptr->cr[i] >> 10) + 1) << 21;
       }
       for (i = 0; i < 8; i++) {
              if (!(tx4925_ebuscptr->ch[i].cr & 0x8))
                              continue;       /* disabled */
              base[i+4] = rbtx4925_ce_base[i];
              size[i+4] = rbtx4925_ce_size[i];
       }
       mbegin = base[0] + size[0];     /* end of SDRAM ch 0 */
       mend = base[4]; /* begin of CE0 */

       mips_pci_mem_base = rbtx4925_find_free_region(sizeof(base) / sizeof(base[0]), base, size, mbegin, mend, &mips_pci_mem_size);
       base[4+8] = mips_pci_mem_base;
       size[4+8] = mips_pci_mem_size;

       mips_pci_io_base = rbtx4925_find_free_region(sizeof(base) / sizeof(base[0]), base, size, mbegin, mend, &mips_pci_io_size);
}

extern struct resource pci_io_resource;
extern struct resource pci_mem_resource;
void
tx4925_pci_setup(void)
{
	static int called = 0;
	extern unsigned int tx4925_get_mem_size(void);

	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_PCI2, "-\n");

        rbtx4925_setup_pci_regions();

       /* setup PCI resources after region setup */
       pci_io_resource.start = PCIBIOS_MIN_IO;
       pci_io_resource.end = PCIBIOS_MIN_IO + mips_pci_io_size - 1;   
       pci_mem_resource.start = mips_pci_mem_base;
       pci_mem_resource.end = mips_pci_mem_base + mips_pci_mem_size - 1;

	mips_memory_upper = tx4925_get_mem_size() << 20;
	mips_memory_upper += KSEG0;
	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_PCI2,
				       "0x%08lx=mips_memory_upper\n",
				       mips_memory_upper);
	if (tx4925_memory_upper_max
	    && mips_memory_upper > tx4925_memory_upper_max) {
		mips_memory_upper = tx4925_memory_upper_max;
		printk("limiting memory_upper to 0x%08lx\n", mips_memory_upper);
	}

	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_PCI2,
				       "0x%08lx=mips_pci_io_base\n",
				       mips_pci_io_base);
	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_PCI2,
				       "0x%08lx=mips_pci_io_size\n",
				       mips_pci_io_size);
	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_PCI2,
				       "0x%08lx=mips_pci_mem_base\n",
				       mips_pci_mem_base);
	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_PCI2,
				       "0x%08lx=mips_pci_mem_size\n",
				       mips_pci_mem_size);
	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_PCI2,
				       "0x%08lx=pci_io_resource.start\n",
				       pci_io_resource.start);
	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_PCI2,
				       "0x%08lx=pci_io_resource.end\n",
				       pci_io_resource.end);
	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_PCI2,
				       "0x%08lx=pci_mem_resource.start\n",
				       pci_mem_resource.start);
	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_PCI2,
				       "0x%08lx=pci_mem_resource.end\n",
				       pci_mem_resource.end);
	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_PCI2,
				       "0x%08lx=mips_io_port_base",
				       mips_io_port_base);

	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_PCI2,
				       "setup pci_io_resource  to 0x%08lx 0x%08lx\n",
				       pci_io_resource.start,
				       pci_io_resource.end);
	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_PCI2,
				       "setup pci_mem_resource to 0x%08lx 0x%08lx\n",
				       pci_mem_resource.start,
				       pci_mem_resource.end);

	if (!called) {
		printk("TX4925 PCIC -- DID:%04x VID:%04x RID:%02x Arbiter:%s\n",
		       (unsigned short) (tx4925_pcicptr->pciid >> 16),
		       (unsigned short) (tx4925_pcicptr->pciid & 0xffff),
		       (unsigned short) (tx4925_pcicptr->pciccrev & 0xff),
		       (tx4925_ccfgptr->
			ccfg & TX4925_CONFIG_CCFG_PCIARB) ? "External" :
		       "Internal");
		called = 1;
	}
	printk("TX4925 PCIC -- PCICLK:");

	/* GB->PCI mappings */
	tx4925_pcicptr->g2piomask = (mips_pci_io_size - 1);
	tx4925_pcicptr->g2piogbase = mips_pci_io_base;
	tx4925_pcicptr->g2piopbase = 0;

	tx4925_pcicptr->g2pm0mask = (mips_pci_mem_size - 1);
	tx4925_pcicptr->g2pm0gbase = mips_pci_mem_base;
	tx4925_pcicptr->g2pm0pbase = mips_pci_mem_base;

	tx4925_pcicptr->g2pm1mask = 0;
	tx4925_pcicptr->g2pm1gbase = 0;
	tx4925_pcicptr->g2pm2pbase = 0;
	tx4925_pcicptr->g2pm2mask = 0;
	tx4925_pcicptr->g2pm2gbase = 0;
	tx4925_pcicptr->g2pm2pbase = 0;

	/* PCI->GB mappings (I/O) */
	tx4925_pcicptr->p2giopbase = 0;	/* 256B */
	tx4925_pcicptr->p2giogbase = 0;

	tx4925_pcicptr->p2gioctr =
#ifdef TX4925_SUPPORT_COMMAND_IO
	    TX4925_PCIC_P2GIOCTR_P2GIOEN
#ifdef __BIG_ENDIAN
	    | TX4925_PCIC_P2GIOCTR_BSWAP
#endif
	    ;
#else
	    0;
#endif

	/* PCI->GB mappings (MEMORY) */
	tx4925_pcicptr->p2gm0pbase = 0;
	tx4925_pcicptr->p2gm0gbase = 0;
	tx4925_pcicptr->p2gm0ctr =
	    ((mips_memory_upper - 1) & TX4925_PCIC_P2GMnCTR_AM_MASK)
	    | TX4925_PCIC_P2GMnCTR_TPRBL_8DW
	    | TX4925_PCIC_P2GMnCTR_MEMnPE | TX4925_PCIC_P2GMnCTR_P2GMnEN
#ifdef __BIG_ENDIAN
	    | TX4925_PCIC_P2GMnCTR_BSWAP
#endif
	    ;

	tx4925_pcicptr->p2gm1ctr = 0;
	tx4925_pcicptr->p2gm2ctr = 0;

	/* Enable Initiator Memory 0 Space, I/O Space, Config */
	tx4925_pcicptr->g2pcfg =
	    TX4925_PCIC_G2PCFG_G2PM0EN | TX4925_PCIC_G2PCFG_G2PIOEN
#ifdef __BIG_ENDIAN
	    | TX4925_PCIC_G2PCFG_BSWAPM0
	    | TX4925_PCIC_G2PCFG_BSWAPM1
	    | TX4925_PCIC_G2PCFG_BSWAPM2 | TX4925_PCIC_G2PCFG_BSWAPIO
// needs to be off                | TX4925_PCIC_G2PCFG_BSWAPI
#endif
	    ;

	tx4925_pcicptr->pciccfg &= TX4925_PCIC_PCICCFG_GBWC_MASK;
	tx4925_pcicptr->pcicfg1 = 0;

	if (tx4925_pcic_trdyto >= 0) {
		tx4925_pcicptr->g2ptocnt &= ~0xff;
		tx4925_pcicptr->g2ptocnt |= (tx4925_pcic_trdyto & 0xff);
	}

	/* Clear All Local Bus Status */
	tx4925_pcicptr->pcicstatus = TX4925_PCIC_PCICSTATUS_ALL;
	/* Enable All Local Bus Interrupts */
	tx4925_pcicptr->pcicmask = TX4925_PCIC_PCICSTATUS_ALL;
	/* Clear All Initiator Status */
	tx4925_pcicptr->g2pstatus = TX4925_PCIC_G2PSTATUS_ALL;
	/* Enable All Initiator Interrupts */
	tx4925_pcicptr->g2pmask = TX4925_PCIC_G2PSTATUS_ALL;
	tx4925_pcicptr->g2pmask &= ~TX4925_PCIC_G2PSTATUS_MDFE;
	/* Clear All PCI Status Error */
	tx4925_pcicptr->pcistatus =
	    (tx4925_pcicptr->pcistatus & 0x0000ffff) |
	    (TX4925_PCIC_PCISTATUS_ALL << 16);
	/* Enable All PCI Status Error Interrupts */
	tx4925_pcicptr->pcimask = TX4925_PCIC_PCISTATUS_ALL;

	/* PCIC Int => IRC IRQ */
	tx4925_pcicptr->pcicfg2 =
	    (tx4925_pcicptr->pcicfg2 & 0xffffff00) | TX4925_IRQ_IRC_PCIC;

	if (tx4925_ccfgptr->ccfg & TX4925_CONFIG_CCFG_PCIARB) {
		/* XXX */
	} else {
		/* Reset Bus Arbiter */
		tx4925_pcicptr->pbacfg = TX4925_PCIC_PBACFG_RPBA;
		/* Enable Bus Arbiter */
		tx4925_pcicptr->pbacfg = TX4925_PCIC_PBACFG_PBAEN;
	}

	tx4925_pcicptr->pcistatus = PCI_COMMAND_MASTER | PCI_COMMAND_MEMORY |
#ifdef TX4925_SUPPORT_COMMAND_IO
	    PCI_COMMAND_IO |
#endif
	    PCI_COMMAND_PARITY | PCI_COMMAND_SERR;

	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_PCI2,
				       ":pci setup complete:\n");
	//tx4925_dump_pcic_settings();

	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_PCI2, "+\n");
	return;
}
#endif				/* CONFIG_PCI */

void
toshiba_rbtx4925_restart(char *command)
{
	printk(KERN_NOTICE "System Rebooting...\n");

	/* enable the s/w reset register */
	reg_wr08(RBTX4925_SW_RESET_ENABLE, RBTX4925_SW_RESET_ENABLE_SET);

	/* wait for enable to be seen */
	while ((reg_rd08(RBTX4925_SW_RESET_ENABLE) &
		RBTX4925_SW_RESET_ENABLE_SET) == 0x00) ;

	/* do a pci s/w reset */
	reg_wr08(RBTX4925_SW_RESET_PCI, RBTX4925_SW_RESET_PCI_SET);

	/* do a mpu s/w reset */
	reg_wr08(RBTX4925_SW_RESET_DO, RBTX4925_SW_RESET_DO_SET);

	/* do something passive while waiting for reset */
	cli();
	while (1) {
		asm_wait();
	}

	/* no return */
}

void
toshiba_rbtx4925_halt(void)
{
	printk(KERN_NOTICE "System Halted\n");
	cli();
	while (1) {
		asm_wait();
	}
	/* no return */
}

void
toshiba_rbtx4925_power_off(void)
{
	toshiba_rbtx4925_halt();
	/* no return */
}

void __init
toshiba_rbtx4925_setup(void)
{
	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_SETUP, "-\n");

	/* f/w leaves this on at startup */
	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_SETUP,
				       ":Clearing STO_ERL.\n");
	clear_cp0_status(ST0_ERL);

#ifdef TOSHIBA_RBTX4925_SETUP_DEBUG
	{
		extern void dump_cp0(char *);
		dump_cp0("toshiba_rbtx4925_early_fw_fixup");
	}
#endif

	/* setup irq stuff */
	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_SETUP,
				       ":Setting up tx4925 pic.\n");
	TX4925_WR(TX4925_MKA(TX4925_IRC_IRDM0), 0x00000040);	/* irq trigger */
	TX4925_WR(TX4925_MKA(TX4925_IRC_IRDM1), 0x00000000);	/* irq trigger */

	/* setup serial stuff */
	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_SETUP,
				       ":Setting up tx4925 sio.\n");
	TX4925_WR(0xff1ff314, 0x00000000);	/* h/w flow control off */
	TX4925_WR(0xff1ff414, 0x00000000);	/* h/w flow control off */

	set_io_port_base(KSEG1 + TBTX4925_ISA_IO_OFFSET);
	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_SETUP,
				       ":mips_io_port_base=0x%08lx\n",
				       mips_io_port_base);

	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_SETUP,
				       ":Resource\n");
	ioport_resource.start = 0x1000;
	ioport_resource.end = 0xffffffff;
	iomem_resource.start = 0x1000;
	iomem_resource.end = 0xffffffff;

	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_SETUP,
				       ":ResetRoutines\n");
	_machine_restart = toshiba_rbtx4925_restart;
	_machine_halt = toshiba_rbtx4925_halt;
	_machine_power_off = toshiba_rbtx4925_power_off;

#ifdef CONFIG_PC_KEYB
	{
		extern struct kbd_ops std_kbd_ops;
		kbd_ops = &std_kbd_ops;
		TOSHIBA_RBTX4925_SETUP_DPRINTK
		    (TOSHIBA_RBTX4925_SETUP_SETUP, ":kbd_ops=&std_kbd_ops\n");
	}
#else
	{
		TOSHIBA_RBTX4925_SETUP_DPRINTK
		    (TOSHIBA_RBTX4925_SETUP_SETUP, ":kbd_ops=<NOT_CONFIG>\n");
	}
#endif

#ifdef CONFIG_BLK_DEV_IDEPCI
	{
		TOSHIBA_RBTX4925_SETUP_DPRINTK
		    (TOSHIBA_RBTX4925_SETUP_SETUP,
		     ":ide_ops=&std_ide_ops(modified)\n");
		ide_ops = &std_ide_ops;
	}
#else
	{
		TOSHIBA_RBTX4925_SETUP_DPRINTK
		    (TOSHIBA_RBTX4925_SETUP_SETUP, ":ide_ops=<NOT_CONFIG>\n");
	}
#endif

#ifdef CONFIG_FB
	{
		conswitchp = &dummy_con;
	}
#endif

#ifdef CONFIG_PCI

	/* PCIC */
#define TOSHIBA_RBTX4925_SETUP_PCI1        ( 1 <<  8 )
	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_PCI1,
				       "PCI mode is %lx, pci arb is %lx\n",
				       (unsigned long) tx4925_ccfgptr->
				       ccfg & TX4925_CONFIG_CCFG_PCIMODE,
				       (unsigned long) tx4925_ccfgptr->
				       ccfg & TX4925_CONFIG_CCFG_PCIARB);

	/* CCFG */
	/* check if enable Timeout BusError */
	if (tx4925_ccfg_toeon)
		tx4925_ccfgptr->ccfg |= TX4925_CONFIG_CCFG_TOE;

	tx4925_pci_setup();
#endif				/* CONFIG_PCI */

	/* this is on ISA bus behind PCI bus, so need PCI up first */
#ifdef CONFIG_TOSHIBA_FPCIB0
	{
		TOSHIBA_RBTX4925_SETUP_DPRINTK
		    (TOSHIBA_RBTX4925_SETUP_SETUP, ":fpcibo=yes\n");

		TOSHIBA_RBTX4925_SETUP_DPRINTK
		    (TOSHIBA_RBTX4925_SETUP_SETUP, ":smsc_fdc37m81x_init()\n");
		smsc_fdc37m81x_init(0x3f0);

		TOSHIBA_RBTX4925_SETUP_DPRINTK
		    (TOSHIBA_RBTX4925_SETUP_SETUP,
		     ":smsc_fdc37m81x_config_beg()\n");
		smsc_fdc37m81x_config_beg();

		TOSHIBA_RBTX4925_SETUP_DPRINTK
		    (TOSHIBA_RBTX4925_SETUP_SETUP,
		     ":smsc_fdc37m81x_config_set(KBD)\n");
		smsc_fdc37m81x_config_set(SMSC_FDC37M81X_DNUM,
					  SMSC_FDC37M81X_KBD);
		smsc_fdc37m81x_config_set(SMSC_FDC37M81X_INT, 1);
		smsc_fdc37m81x_config_set(SMSC_FDC37M81X_INT2, 12);
		smsc_fdc37m81x_config_set(SMSC_FDC37M81X_ACTIVE, 1);

		smsc_fdc37m81x_config_end();
		TOSHIBA_RBTX4925_SETUP_DPRINTK
		    (TOSHIBA_RBTX4925_SETUP_SETUP,
		     ":smsc_fdc37m81x_config_end()\n");
	}
#else
	{
		TOSHIBA_RBTX4925_SETUP_DPRINTK
		    (TOSHIBA_RBTX4925_SETUP_SETUP, ":fpcibo=no\n");
	}
#endif

	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_SETUP, "+\n");

	return;
}

void __init
toshiba_rbtx4925_time_init(void)
{
	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_TIME_INIT, "-\n");

#if 0
	rtc_get_time = tx4925_rtc_get_time;
	rtc_set_time = tx4925_rtc_set_time;

	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_TIME_INIT,
				       ":tx4925_rtc_init()-\n");
	tx4925_rtc_init(NULL);
	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_TIME_INIT,
				       ":tx4925_rtc_init()+\n");

	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_TIME_INIT,
				       ":Calibrate mips_counter_frequency-\n");
	tx4925_rtc_wait();

	/* get the count */
	c1 = read_32bit_cp0_register(CP0_COUNT);

	/* wait for the seconds to change again */
	tx4925_rtc_wait();

	/* get the count again */
	c2 = read_32bit_cp0_register(CP0_COUNT);

	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_TIME_INIT,
				       ":Calibrate mips_counter_frequency+\n");
	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_TIME_INIT,
				       ":c1=%12u\n", c1);
	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_TIME_INIT,
				       ":c2=%12u\n", c2);

	/* this diff is as close as we are going to get to counter ticks per sec */
	mips_counter_frequency = abs(c2 - c1);
	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_TIME_INIT,
				       ":f1=%12u\n", mips_counter_frequency);

	/* round to 1/10th of a MHz */
	mips_counter_frequency /= (100 * 1000);
	mips_counter_frequency *= (100 * 1000);
	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_TIME_INIT,
				       ":f2=%12u\n", mips_counter_frequency);

	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_INFO,
				       ":mips_counter_frequency=%uHz (%uMHz)\n",
				       mips_counter_frequency,
				       mips_counter_frequency / 1000000);
#else
	mips_counter_frequency = 100000000;
#endif

	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_TIME_INIT, "+\n");

	return;
}

void __init
toshiba_rbtx4925_timer_setup(struct irqaction *irq)
{
	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_TIMER_SETUP,
				       "-\n");
	TOSHIBA_RBTX4925_SETUP_DPRINTK(TOSHIBA_RBTX4925_SETUP_TIMER_SETUP,
				       "+\n");
	return;
}
