/*
 *  Cache test for MPU-22x(TX49x7)
 *
 *  Copyright 2003 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/config.h>
#include <linux/version.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/fs.h>
#include <linux/types.h>
#include <linux/interrupt.h>
#include <linux/sched.h>
#include <linux/irq.h>
#include <linux/proc_fs.h>
#include <linux/seq_file.h>
#include <asm/system.h>
#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/cacheops.h>

MODULE_AUTHOR("Sony NSC");
MODULE_LICENSE("GPL");

#define MODNAME             "MPU22x_CTEST"

/* proc */
#define DC_PROC_NAME        "dctag"
#define IC_PROC_NAME        "ictag"

static spinlock_t ctest_lock = SPIN_LOCK_UNLOCKED;

#define DUMP_CACHE_BASE     0x80000000
#define DUMP_CACHE_SIZE     0x2000

#define Fill_I                  0x14
#define Fill_D                  0x1d

#define CACHE_CMD(addr, ope)                        \
	__asm__ __volatile__ (                      \
		".set push\n"                       \
		".set mips3\n"                      \
		"cache %1, (%0)\n"                  \
		"nop\n"                             \
		".set mips0\n"                      \
		".set pop"                          \
		: : "r" (addr), "i" (ope))


/*
 *  lock a certail way of cache
 */
static inline void 
dcache_lock_way(u_int32_t addr, u_int32_t size, int way)
{
	u_int32_t val;

	set_taghi(0);
	for (size += addr; addr < size; addr += 32) {
		val = ((addr & 0x1ffff000) >> 4) | 0x04;
		set_taglo(val);
		CACHE_CMD(addr | way, Index_Store_Tag_D);
		CACHE_CMD(addr | way, Fill_D);
	}
}


static inline void 
icache_lock_way(u_int32_t addr, u_int32_t size, int way)
{
	u_int32_t val;

	set_taghi(0);
	for (size += addr; addr < size; addr += 32) {
		val = ((addr & 0x1ffff000) >> 4) | 0x04;
		set_taglo(val);
		CACHE_CMD(addr | way, Index_Store_Tag_I);
		CACHE_CMD(addr | way, Fill_I);
	}
}

/*
 *  lock 2way of D$ and/or I$ and pretend to have half size of cache
 */
void
mpu22x_pretend_halfcache(void)
{
	/* assume that cache is disabled and flushed */
#ifdef CONFIG_SNSC_MPU22X_HALF_DCACHE
	dcache_lock_way(0x96f00000, 0x2000, 2);
	dcache_lock_way(0x96f40000, 0x2000, 3);
#endif
#ifdef CONFIG_SNSC_MPU22X_HALF_ICACHE
	icache_lock_way(0x96f80000, 0x2000, 2);
	icache_lock_way(0x96fc0000, 0x2000, 3);
#endif
}


/*
 *  proc I/F for dump cache tags
 */
static void *
dump_seq_start(struct seq_file *m, loff_t *pos)
{
	u_int32_t addr;

	addr = DUMP_CACHE_BASE + ((*pos) << 5);
	spin_lock_irq(&ctest_lock);
	if (addr < (DUMP_CACHE_BASE + DUMP_CACHE_SIZE))
		return (void *)addr;
	else {
		spin_unlock_irq(&ctest_lock);
		return NULL;
	}
}

static void *
dump_seq_next(struct seq_file *m, void *v, loff_t *pos)
{
	u_int32_t addr;

	(*pos)++;
	addr = DUMP_CACHE_BASE + ((*pos) << 5);
	if (addr < (DUMP_CACHE_BASE + DUMP_CACHE_SIZE))
		return (void *)addr;
	else {
		spin_unlock_irq(&ctest_lock);
		return NULL;
	}
}


static void
dump_seq_stop(struct seq_file *m, void *v)
{
	if (v) {
		spin_unlock_irq(&ctest_lock);
	}
}

static int
dcdump_seq_show(struct seq_file *m, void *v)
{
	u_int32_t addr = (u_int32_t)v;
	u_int32_t val;
	int way;

	if (addr == DUMP_CACHE_BASE)
		seq_printf(m, "D$ tags :\n");

	seq_printf(m, "0x%02x : ", (addr >> 5) & 0xff);
	for (way = 0; way < 4; way++) {
		CACHE_CMD(addr | way, Index_Load_Tag_D);
		val = get_taglo();
		seq_printf(m, "%08x(", val);
		if ((val & 0xc0) == 0xc0)
			seq_printf(m, "V");
		else
			seq_printf(m, " ");
		if ((val & 4))
			seq_printf(m, "L");
		else
			seq_printf(m, " ");
		seq_printf(m, ") ");
	}
	seq_printf(m, "| (TagHI)0x%08x\n", (u_int32_t)get_taghi());
	return 0;
}

static int
icdump_seq_show(struct seq_file *m, void *v)
{
	u_int32_t addr = (u_int32_t)v;
	u_int32_t val;
	int way;

	if (addr == DUMP_CACHE_BASE)
		seq_printf(m, "I$ tags :\n");

	seq_printf(m, "0x%02x : ", (addr >> 5) & 0xff);
	for (way = 0; way < 4; way++) {
		CACHE_CMD(addr | way, Index_Load_Tag_I);
		val = get_taglo();
		seq_printf(m, "%08x(", val);
		if ((val & 0xc0) == 0xc0)
			seq_printf(m, "V");
		else
			seq_printf(m, " ");
		if ((val & 4))
			seq_printf(m, "L");
		else
			seq_printf(m, " ");
		seq_printf(m, ") ");
	}
	seq_printf(m, "| (TagHI)0x%08x\n", (u_int32_t)get_taghi());
	return 0;
}

static struct seq_operations proc_dctag_seqop = {
    start:  dump_seq_start,
    next:   dump_seq_next,
    stop:   dump_seq_stop,
    show:   dcdump_seq_show
};

static struct seq_operations proc_ictag_seqop = {
    start:  dump_seq_start,
    next:   dump_seq_next,
    stop:   dump_seq_stop,
    show:   icdump_seq_show
};

static int proc_dctag_open(struct inode *inode, struct file *file)
{
    return seq_open(file, &proc_dctag_seqop);
}

static int proc_ictag_open(struct inode *inode, struct file *file)
{
    return seq_open(file, &proc_ictag_seqop);
}

static struct file_operations proc_dctag_operations = {
    open:           proc_dctag_open,
    read:           seq_read,
    llseek:         seq_lseek,
    release:        seq_release,
};

static struct file_operations proc_ictag_operations = {
    open:           proc_ictag_open,
    read:           seq_read,
    llseek:         seq_lseek,
    release:        seq_release,
};

static struct proc_dir_entry *proc_dc, *proc_ic;

static int __init
mpu22x_ctest_init(void)
{
	if (!(proc_dc = create_proc_entry(DC_PROC_NAME, 0444, NULL))) {
		printk(MODNAME ": unable to create proc entry %s\n",
		       DC_PROC_NAME);
		return -ENOMEM;
	}
	proc_dc->proc_fops = &proc_dctag_operations;
	if (!(proc_ic = create_proc_entry(IC_PROC_NAME, 0444, NULL))) {
		printk(MODNAME ": unable to create proc entry %s\n",
		       IC_PROC_NAME);
		return -ENOMEM;
	}
	proc_ic->proc_fops = &proc_ictag_operations;

	return 0;
}

static void __exit
mpu22x_ctest_exit(void)
{
	remove_proc_entry(DC_PROC_NAME, NULL);
	remove_proc_entry(IC_PROC_NAME, NULL);
}

module_init(mpu22x_ctest_init);
module_exit(mpu22x_ctest_exit);
