/*
 * linux/arch/mips/tx4927/snsc_mpu22x/snsc_mpu22x_irq.c
 *
 * Sony NSC MPU-220 specific interrupt handlers
 *
 * Copyright 2003 Sony Corporation.
 *
 * This program is based on linux/arch/mips/tx4927/toshiba_rbtx4927/toshiba_rbtx4927_irq.c
 *
 * linux/arch/mips/tx4927/toshiba_rbtx4927/toshiba_rbtx4927_irq.c
 *
 * Toshiba RBTX4927 specific interrupt handlers
 *
 * Author: MontaVista Software, Inc.
 *         source@mvista.com
 *
 * Copyright 2001-2002 MontaVista Software Inc.
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 *  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 *  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */


/*
IRQ  Device  
0-15 none

16   TX4927-CP0/00 Software 0
17   TX4927-CP0/01 Software 1
18   TX4927-CP0/02 Cascade TX4927-CP0
19   TX4927-CP0/03 Multiplexed -- do not use 
20   TX4927-CP0/04 Multiplexed -- do not use 
21   TX4927-CP0/05 Multiplexed -- do not use 
22   TX4927-CP0/06 Multiplexed -- do not use 
23   TX4927-CP0/07 CPU TIMER

24   TX4927-PIC/00
25   TX4927-PIC/01
26   TX4927-PIC/02 PCIA 
27   TX4927-PIC/03 PCIB
28   TX4927-PIC/04 PCIC
29   TX4927-PIC/05 PCID
30   TX4927-PIC/06 Cascade CXD9790(IROUT0)
31   TX4927-PIC/07 Cascade CXD9790(IROUT1)
32   TX4927-PIC/08 TX4927 SerialIO Channel 0
33   TX4927-PIC/09 TX4927 SerialIO Channel 1
34   TX4927-PIC/10
35   TX4927-PIC/11
36   TX4927-PIC/12
37   TX4927-PIC/13
38   TX4927-PIC/14
39   TX4927-PIC/15
40   TX4927-PIC/16 TX4927 PCI PCI-C
41   TX4927-PIC/17
42   TX4927-PIC/18
43   TX4927-PIC/19
44   TX4927-PIC/20
45   TX4927-PIC/21
46   TX4927-PIC/22 TX4927 PCI PCI-ERR 
47   TX4927-PIC/23 TX4927 PCI PCI-PMA (not used)
48   TX4927-PIC/24
49   TX4927-PIC/25
50   TX4927-PIC/26
51   TX4927-PIC/27
52   TX4927-PIC/28
53   TX4927-PIC/29
54   TX4927-PIC/30
55   TX4927-PIC/31

56   CXD9790-IRCS/0 UART0
57   CXD9790-IRCS/1 UART1
58   CXD9790-IRCS/2 UART2
59   CXD9790-IRCS/3 UART3
60   CXD9790-IRCS/4 I2C
61   CXD9790-IRCS/5 GPIO C/D
62   CXD9790-IRCS/6 GPIO A/B
63   CXD9790-IRCS/7 GPIO Z 

64-71 CXD9790-GPIOZ/0-7  GPIOZ[7:0]
72-79 CXD9790-GPIOA/0-7  GPIOA[7:0]
80-87 CXD9790-GPIOB/0-7  GPIOB[7:0]
88-95 CXD9790-GPIOC/0-7  GPIOC[7:0]
96-103 CXD9790-GPIOD/0-7  GPIOD[7:0]
*/

#include <linux/config.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/mm.h>
#include <linux/swap.h>
#include <linux/ioport.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/pci.h>
#include <linux/timex.h>
#include <asm/bootinfo.h>
#include <asm/page.h>
#include <asm/bootinfo.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/pci.h>
#include <asm/processor.h>
#include <asm/ptrace.h>
#include <asm/reboot.h>
#include <asm/time.h>
#include <linux/version.h>
#include <linux/bootmem.h>
#include <linux/blk.h>
#include <linux/tx4927_sio.h>
#include <asm/tx4927/snsc_mpu22x/snsc_mpu22x.h>
#ifdef CONFIG_SNSC_CXD9790_INTR
#include <linux/cxd9790.h>
#endif


#undef SNSC_MPU22X_IRQ_DEBUG

#ifdef SNSC_MPU22X_IRQ_DEBUG
#define SNSC_MPU22X_IRQ_NONE        0x00000000

#define SNSC_MPU22X_IRQ_INFO          ( 1 <<  0 )
#define SNSC_MPU22X_IRQ_WARN          ( 1 <<  1 )
#define SNSC_MPU22X_IRQ_EROR          ( 1 <<  2 )

#define SNSC_MPU22X_SETUP_ALL         0xffffffff
#endif


#ifdef SNSC_MPU22X_IRQ_DEBUG
static const u32 snsc_mpu22x_irq_debug_flag =
    (SNSC_MPU22X_IRQ_NONE | SNSC_MPU22X_IRQ_INFO |
     SNSC_MPU22X_IRQ_WARN | SNSC_MPU22X_IRQ_EROR );
#endif


#ifdef SNSC_MPU22X_IRQ_DEBUG
#define SNSC_MPU22X_IRQ_DPRINTK(flag,str...) \
        if ( (snsc_mpu22x_irq_debug_flag) & (flag) ) \
        { \
           char tmp[100]; \
           sprintf( tmp, str ); \
           printk( "%s(%s:%u)::%s", __FUNCTION__, __FILE__, __LINE__, tmp ); \
        }
#else
#define SNSC_MPU22X_IRQ_DPRINTK(flag,str...)
#endif

void __init init_IRQ(void)
{
	extern void tx4927_irq_init(void);
#ifdef CONFIG_PCI
	extern void snsc_mpu22x_pci_irq_init(void);
#endif

	cli();

	tx4927_irq_init();
#ifdef CONFIG_PCI
	snsc_mpu22x_pci_irq_init();
#endif

#ifdef CONFIG_SNSC_CXD9790_INTR
	/* initialize CXD9790 IRQs */
	if (!cxd9790_irq_init()) {
		setup_irq(MPU22X_IRQ_CXD9790_OUT0, &cxd9790_ircs_irq_action);
		setup_irq(MPU22X_IRQ_CXD9790_OUT1, &cxd9790_ircs_irq_action);
	}
#endif /* CONFIG_SNSC_CXD9790_INTR */

	wbflush();

	return;
}

/*
 *  get CXD9790 IRQ number 
 */
int snsc_mpu22x_irq_nested0(int sw_irq)
{
#ifdef CONFIG_SNSC_CXD9790_INTR
	int irq;

	irq = cxd9790_get_irq(IROUT0);
	if (irq < 0)
		return sw_irq;
	return irq;
#else
	return sw_irq;
#endif
}

int snsc_mpu22x_irq_nested1(int sw_irq)
{
#ifdef CONFIG_SNSC_CXD9790_INTR
	int irq;

	irq = cxd9790_get_irq(IROUT1);
	if (irq < 0)
		return sw_irq;
	return irq;
#else
	return sw_irq;
#endif
}

void snsc_mpu22x_irq_dump(char *key)
{
#ifdef SNSC_MPU22X_IRQ_DEBUG
	{
		u32 i, j = 0;
		for (i = 0; i < NR_IRQS; i++) {
			if (strcmp(irq_desc[i].handler->typename, "none")
			    == 0)
				continue;

			if ((i >= 1)
			    && (irq_desc[i - 1].handler->typename ==
				irq_desc[i].handler->typename)) {
				j++;
			} else {
				j = 0;
			}
			SNSC_MPU22X_IRQ_DPRINTK
			    (SNSC_MPU22X_IRQ_INFO,
			     "%s irq=0x%02x/%3d s=0x%08x h=0x%08x a=0x%08x ah=0x%08x d=%1d n=%s/%02d\n",
			     key, i, i, irq_desc[i].status,
			     (u32) irq_desc[i].handler,
			     (u32) irq_desc[i].action,
			     (u32) (irq_desc[i].action ? irq_desc[i].
				    action->handler : 0),
			     irq_desc[i].depth,
			     irq_desc[i].handler->typename, j);
		}
	}
#endif
	return;
}
