/*
 * linux/arch/mips/tx4927/snsc_mpu22x/snsc_mpu22x_prom.c
 *
 * Sony NSC MPU-22X prom routines
 *
 * Copyright 2003 Sony Corporation.
 *
 * This program is based on linux/arch/mips/tx4927/toshiba_rbtx4927/toshiba_rbtx4927_prom.c
 *
 * linux/arch/mips/tx4927/toshiba_rbtx4927/toshiba_rbtx4927_prom.c
 *
 * rbtx4927 specific prom routines
 *
 * Author: MontaVista Software, Inc.
 *         source@mvista.com
 *
 * Copyright 2001-2002 MontaVista Software Inc.
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 *  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 *  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/init.h>
#include <linux/mm.h>
#include <linux/sched.h>
#include <linux/bootmem.h>

#include <asm/addrspace.h>
#include <asm/bootinfo.h>
#include <asm/tx4927/tx4927.h>
#include <asm/tx4927/snsc_mpu22x/snsc_mpu22x.h>

#ifndef CONFIG_CMDLINE_BOOL
#define CONFIG_CMDLINE ""
#endif

#define CONFIG_REVID               (0xff1fe008)
#define        REVID_PCODE(x)              (((x) >> 16) & 0xffff)

#ifndef COMMAND_LINE_SIZE
#define COMMAND_LINE_SIZE CL_SIZE
#endif

char arcs_cmdline[COMMAND_LINE_SIZE] = CONFIG_CMDLINE;

int mpu22x_cputype;

void __init prom_init_cmdline(int argc, char **argv)
{
	int i;			/* Always ignore the "-c" at argv[0] */

	/* ignore all built-in args if any f/w args given */
	if (argc > 1) {
		*arcs_cmdline = '\0';
	}

	for (i = 1; i < argc; i++) {
		if (i != 1) {
			strcat(arcs_cmdline, " ");
		}
		strcat(arcs_cmdline, argv[i]);
	}
}


void __init prom_init(int argc, char **argv, char **envp, int *pvec)
{
	extern int tx4927_get_mem_size(void);
	int msize;
        u_int32_t rev;

	prom_init_cmdline(argc, argv);

	mips_machgroup = MACH_GROUP_TOSHIBA;
	mips_machtype = MACH_SNSC_MPU22X;

	msize = tx4927_get_mem_size();
	add_memory_region(0, msize << 20, BOOT_MEM_RAM);

        /* Revision ID */
        rev = TX4927_RD(CONFIG_REVID);
        if (REVID_PCODE(rev) == 0x4927)
                mpu22x_cputype = MPU22X_CPU_TX4927;
        else if (REVID_PCODE(rev) == 0x4937)
                mpu22x_cputype = MPU22X_CPU_TX4937;
        else
                mpu22x_cputype = MPU22X_CPU_TX_UNKNOWN;

	return;
}


void __init prom_free_prom_memory(void)
{
	return;
}


void __init prom_fixup_mem_map(unsigned long start, unsigned long end)
{
	return;
}

const char *get_system_type(void)
{
	return "Sony NSC MPU-22X";
}
