/*
 * linux/arch/mips/tx4927/snsc_mpu22x/snsc_mpu22x_setup.c
 *
 * Sony NSC MPU-22X board setup.
 *
 * Copyright 2003 Sony Corporation.
 *
 * This program is based on linux/arch/mips/tx4927/toshiba_rbtx4927/toshiba_rbtx4927_setup.c
 *
 * linux/arch/mips/tx4927/toshiba_rbtx4927/toshiba_rbtx4927_setup.c
 *
 * Toshiba rbtx4927 specific setup
 *
 * Author: MontaVista Software, Inc.
 *         source@mvista.com
 *
 * Copyright 2001-2002 MontaVista Software Inc.
 *
 * Copyright (C) 1996, 1997, 2001  Ralf Baechle
 * Copyright (C) 2000 RidgeRun, Inc.
 * Author: RidgeRun, Inc.
 *   glonnon@ridgerun.com, skranz@ridgerun.com, stevej@ridgerun.com
 *
 * Copyright 2001 MontaVista Software Inc.
 * Author: jsun@mvista.com or jsun@junsun.net
 *
 * Copyright 2002 MontaVista Software Inc.
 * Author: Michael Pruznick, michael_pruznick@mvista.com
 *
 * Copyright (C) 2000-2001 Toshiba Corporation 
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 *  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 *  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/config.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/mm.h>
#include <linux/swap.h>
#include <linux/ioport.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/pci.h>
#include <linux/timex.h>
#include <asm/bootinfo.h>
#include <asm/page.h>
#include <asm/bootinfo.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/pci.h>
#include <asm/processor.h>
#include <asm/ptrace.h>
#include <asm/reboot.h>
#include <asm/time.h>
#include <linux/version.h>
#include <linux/bootmem.h>
#include <linux/blk.h>
#include <linux/tx4927_sio.h>
#include <asm/tx4927/tx4927.h>
#include <asm/tx4927/snsc_mpu22x/snsc_mpu22x.h>
#ifdef CONFIG_PCI
#include <asm/tx4927/tx4927_pci.h>
#include <linux/types.h>
#include <linux/pci.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <asm/pci_channel.h>
#endif
#ifdef CONFIG_BLK_DEV_IDEPCI
#include <linux/hdreg.h>
#include <asm/ptrace.h>
#include <linux/ide.h>
extern struct ide_ops std_ide_ops;
#endif
#ifdef CONFIG_SNSC_CXD9790
#include <linux/cxd9790.h>
#include <linux/cxd9790_reg.h>

#define MPU22X_SHTDWNACK_GPIO		CXD9790_GPIOZ
#define MPU22X_SHTDWNACK_GPIO_BITNUM	4
#define MPU22X_SELFRST_GPIO		CXD9790_GPIOZ
#define MPU22X_SELFRST_GPIO_BITNUM	2
#define MPU22X_PWROFF_GPIO		CXD9790_GPIOZ
#define MPU22X_PWROFF_GPIO_BITNUM	0

#define MPU22X_PCPCI2_GPIO		CXD9790_GPIOA
#define MPU22X_PCPCI2_GPIO_BITNUM	0
#define MPU22X_PCPCI3_GPIO		CXD9790_GPIOB
#define MPU22X_PCPCI3_GPIO_BITNUM	4

inline void
my_cxd9790_gpio_outbit(int gpio, int bitnum, int is_set)
{
	volatile unsigned char *reg_po, *reg_poen;

	reg_po = (volatile unsigned char *)
		KSEG1ADDR(CXD9790_BASE + gpio + CXD9790_PO);
	reg_poen = (volatile unsigned char *)
		KSEG1ADDR(CXD9790_BASE + gpio + CXD9790_POEN);

	if (is_set) {
		*reg_po |= (1 << bitnum);
	} else {
		*reg_po &= ~(1 << bitnum);
	}
	*reg_poen |= (1 << bitnum);
}
#endif /* CONFIG_SNSC_CXD9790 */


#undef SNSC_MPU22X_SETUP_DEBUG

#ifdef SNSC_MPU22X_SETUP_DEBUG
#define SNSC_MPU22X_SETUP_NONE        0x00000000

#define SNSC_MPU22X_SETUP_INFO        ( 1 <<  0 )
#define SNSC_MPU22X_SETUP_WARN        ( 1 <<  1 )
#define SNSC_MPU22X_SETUP_EROR        ( 1 <<  2 )

#define SNSC_MPU22X_SETUP_EFWFU       ( 1 <<  3 )
#define SNSC_MPU22X_SETUP_SETUP       ( 1 <<  4 )
#define SNSC_MPU22X_SETUP_TIME_INIT   ( 1 <<  5 )
#define SNSC_MPU22X_SETUP_TIMER_SETUP ( 1 <<  6 )
#define SNSC_MPU22X_SETUP_PCIBIOS     ( 1 <<  7 )
#define SNSC_MPU22X_SETUP_PCI1        ( 1 <<  8 )
#define SNSC_MPU22X_SETUP_PCI2        ( 1 <<  9 )
#define SNSC_MPU22X_SETUP_PCI66       ( 1 << 10 )

#define SNSC_MPU22X_SETUP_ALL         0xffffffff
#endif


#ifdef SNSC_MPU22X_SETUP_DEBUG
static const u32 snsc_mpu22x_setup_debug_flag =
    (SNSC_MPU22X_SETUP_NONE | SNSC_MPU22X_SETUP_INFO |
     SNSC_MPU22X_SETUP_WARN | SNSC_MPU22X_SETUP_EROR |
     SNSC_MPU22X_SETUP_EFWFU | SNSC_MPU22X_SETUP_SETUP |
     SNSC_MPU22X_SETUP_TIME_INIT | SNSC_MPU22X_SETUP_TIMER_SETUP
     | SNSC_MPU22X_SETUP_PCIBIOS | SNSC_MPU22X_SETUP_PCI1 |
     SNSC_MPU22X_SETUP_PCI2 | SNSC_MPU22X_SETUP_PCI66);
#endif


#ifdef SNSC_MPU22X_SETUP_DEBUG
#define SNSC_MPU22X_SETUP_DPRINTK(flag,str...) \
        if ( (snsc_mpu22x_setup_debug_flag) & (flag) ) \
        { \
           char tmp[100]; \
           sprintf( tmp, str ); \
           printk( "%s(%s:%u)::%s", __FUNCTION__, __FILE__, __LINE__, tmp ); \
        }
#else
#define SNSC_MPU22X_SETUP_DPRINTK(flag,str...)
#endif


/* These functions are used for rebooting or halting the machine*/
extern void snsc_mpu22x_restart(char *command);
extern void snsc_mpu22x_halt(void);
extern void snsc_mpu22x_power_off(void);
extern void snsc_mpu22x_irq_setup(void);

#ifdef CONFIG_PCI
#define CONFIG_TX4927BUG_WORKAROUND
#undef TX4927_SUPPORT_COMMAND_IO
#undef  TX4927_SUPPORT_PCI_66
int tx4927_cpu_clock = 100000000;	/* 100MHz */
unsigned long mips_pci_io_base;
unsigned long mips_pci_io_size;
unsigned long mips_pci_mem_base;
unsigned long mips_pci_mem_size;
/* for legacy I/O, PCI I/O PCI Bus address must be 0 */
unsigned long mips_pci_io_pciaddr = 0;
unsigned long mips_memory_upper;
unsigned long tx4927_memory_upper_max = 0;
static int tx4927_ccfg_toeon = 1;
static int tx4927_pcic_retryto = 0;	/* default: disabled */
static int tx4927_pcic_trdyto = 0;	/* default: disabled */
unsigned long tx4927_ce_base[8];
void tx4927_pci_setup(void);
void tx4927_reset_pci_pcic(void);
#ifdef  TX4927_SUPPORT_PCI_66
void tx4927_pci66_setup(void);
extern int tx4927_pci66_check(void);
#endif
int tx4927_pci66 = 0;		/* 0:auto */
#endif

#ifdef CONFIG_PCI
void tx4927_dump_pcic_settings(void)
{
	printk("tx4927 pcic settings:");
	{
		int i;
		unsigned long *preg = (unsigned long *) tx4927_pcicptr;
		for (i = 0; i < sizeof(struct tx4927_pcic_reg); i += 4) {
			if (i % 32 == 0)
				printk("\n%04x:", i);
			if (preg == &tx4927_pcicptr->g2pintack
			    || preg == &tx4927_pcicptr->g2pspc
#ifdef CONFIG_TX4927BUG_WORKAROUND
			    || preg == &tx4927_pcicptr->g2pcfgadrs
			    || preg == &tx4927_pcicptr->g2pcfgdata
#endif
			    ) {
				printk(" XXXXXXXX");
				preg++;
				continue;
			}
			printk(" %08lx", *preg++);
			if (preg == &tx4927_pcicptr->g2pcfgadrs)
				break;
		}
		printk("\n");
	}
}

static void tx4927_pcierr_interrupt(int irq, void *dev_id,
				    struct pt_regs *regs)
{
	extern void tx4927_dump_pcic_settings(void);

#ifdef CONFIG_BLK_DEV_IDEPCI
	/* ignore MasterAbort for ide probing... */
	if (irq == TX4927_IRQ_IRC_PCIERR &&
	    ((tx4927_pcicptr->pcistatus >> 16) & 0xf900) ==
	    PCI_STATUS_REC_MASTER_ABORT) {
		tx4927_pcicptr->pcistatus =
		    (tx4927_pcicptr->
		     pcistatus & 0x0000ffff) | (PCI_STATUS_REC_MASTER_ABORT
						<< 16);

		return;
	}
#endif
	printk("PCI error interrupt (irq 0x%x).\n", irq);
	printk("pcistat:%04x, g2pstatus:%08lx, pcicstatus:%08lx\n",
	       (unsigned short) (tx4927_pcicptr->pcistatus >> 16),
	       tx4927_pcicptr->g2pstatus, tx4927_pcicptr->pcicstatus);
	printk("ccfg:%08lx, tear:%02lx_%08lx\n",
	       (unsigned long) tx4927_ccfgptr->ccfg,
	       (unsigned long) (tx4927_ccfgptr->tear >> 32),
	       (unsigned long) tx4927_ccfgptr->tear);
	show_regs(regs);
	tx4927_dump_pcic_settings();
	panic("PCI error at PC:%08lx.", regs->cp0_epc);
}

static struct irqaction pcic_action = {
	tx4927_pcierr_interrupt, 0, 0, "PCI-C", NULL, NULL
};

static struct irqaction pcierr_action = {
	tx4927_pcierr_interrupt, 0, 0, "PCI-ERR", NULL, NULL
};


void __init snsc_mpu22x_pci_irq_init(void)
{
	setup_irq(TX4927_IRQ_IRC_PCIC, &pcic_action);
	setup_irq(TX4927_IRQ_IRC_PCIERR, &pcierr_action);
	return;
}

void tx4927_reset_pci_pcic(void)
{
	/* Reset PCI Bus */
	*tx4927_pcireset_ptr = 1;
	/* Reset PCIC */
	tx4927_ccfgptr->clkctr |= TX4927_CLKCTR_PCIRST;
	udelay(10000);
	/* clear PCIC reset */
	tx4927_ccfgptr->clkctr &= ~TX4927_CLKCTR_PCIRST;
	*tx4927_pcireset_ptr = 0;
}
#endif				/* CONFIG_PCI */


#ifdef CONFIG_PCI
#ifdef  TX4927_SUPPORT_PCI_66
void tx4927_pci66_setup(void)
{
	int pciclk, pciclkin = 1;

	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_PCI66,
				       "-\n");

	if (tx4927_ccfgptr->ccfg & TX4927_CCFG_PCI66)
		return;

	tx4927_reset_pci_pcic();

	/* Assert M66EN */
	tx4927_ccfgptr->ccfg |= TX4927_CCFG_PCI66;
	/* set PCICLK 66MHz */
	if (tx4927_ccfgptr->pcfg & TX4927_PCFG_PCICLKEN_ALL) {
		unsigned int pcidivmode = 0;
		pcidivmode =
		    (unsigned long) tx4927_ccfgptr->
		    ccfg & TX4927_CCFG_PCIDIVMODE_MASK;
		if (tx4927_cpu_clock >= 170000000) {
			/* CPU 200MHz */
			pcidivmode = TX4927_CCFG_PCIDIVMODE_3;
			pciclk = tx4927_cpu_clock / 3;
		} else {
			/* CPU 166MHz */
			pcidivmode = TX4927_CCFG_PCIDIVMODE_2_5;
			pciclk = tx4927_cpu_clock * 2 / 5;
		}
		tx4927_ccfgptr->ccfg =
		    (tx4927_ccfgptr->ccfg & ~TX4927_CCFG_PCIDIVMODE_MASK)
		    | pcidivmode;
		SNSC_MPU22X_SETUP_DPRINTK
		    (SNSC_MPU22X_SETUP_PCI66,
		     ":PCICLK: ccfg:0x%08lx\n",
		     (unsigned long) tx4927_ccfgptr->ccfg);
	} else {
		int pciclk_setting = *tx4927_pci_clk_ptr;
		pciclkin = 0;
		pciclk = 66666666;
		pciclk_setting &= ~TX4927_PCI_CLK_MASK;
		pciclk_setting |= TX4927_PCI_CLK_66;
		*tx4927_pci_clk_ptr = pciclk_setting;
		SNSC_MPU22X_SETUP_DPRINTK
		    (SNSC_MPU22X_SETUP_PCI66,
		     "PCICLK: pci_clk:%02x\n", *tx4927_pci_clk_ptr);
	}

	udelay(10000);

	/* clear PCIC reset */
	tx4927_ccfgptr->clkctr &= ~TX4927_CLKCTR_PCIRST;
	/* clear PCI reset */
	*tx4927_pcireset_ptr = 0;

	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_PCI66,
				       "+\n");
	return;
}
#endif				/* TX4927_SUPPORT_PCI_66 */

void print_pci_status(void)
{
	printk("PCI STATUS %lx\n", tx4927_pcicptr->pcistatus);
	printk("PCIC STATUS %lx\n", tx4927_pcicptr->pcicstatus);
}


static struct pci_dev *fake_pci_dev(struct pci_channel *hose,
				    int top_bus, int busnr, int devfn)
{
	static struct pci_dev dev;
	static struct pci_bus bus;

	dev.bus = &bus;
	dev.sysdata = hose;
	dev.devfn = devfn;
	bus.number = busnr;
	bus.ops = hose->pci_ops;

	if (busnr != top_bus)
		/* Fake a parent bus structure. */
		bus.parent = &bus;
	else
		bus.parent = NULL;

	return &dev;
}

#if 0
#define EARLY_PCI_OP(rw, size, type)                                    \
static int early_##rw##_config_##size(struct pci_channel *hose,                \
        int top_bus, int bus, int devfn, int offset, type value)        \
{                                                                       \
        return pci_##rw##_config_##size(                                \
                fake_pci_dev(hose, top_bus, bus, devfn),                \
                offset, value);                                         \
}
EARLY_PCI_OP(read, byte, u8 *)
    EARLY_PCI_OP(read, word, u16 *)
    EARLY_PCI_OP(read, dword, u32 *)
    EARLY_PCI_OP(write, byte, u8)
    EARLY_PCI_OP(write, word, u16)
    EARLY_PCI_OP(write, dword, u32)
#endif

extern struct resource pci_io_resource;
extern struct resource pci_mem_resource;
void tx4927_pci_setup(void)
{
	static int called = 0;
	extern unsigned int tx4927_get_mem_size(void);

	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_PCI2, "-\n");

#ifndef  TX4927_SUPPORT_PCI_66
	if (tx4927_ccfgptr->ccfg & TX4927_CCFG_PCI66)
		printk("PCI 66 current unsupported\n");
#endif

	mips_memory_upper = tx4927_get_mem_size() << 20;
	mips_memory_upper += KSEG0;
	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_PCI2,
				       "0x%08lx=mips_memory_upper\n",
				       mips_memory_upper);
	if (tx4927_memory_upper_max
	    && mips_memory_upper > tx4927_memory_upper_max) {
		mips_memory_upper = tx4927_memory_upper_max;
		printk("limiting memory_upper to 0x%08lx\n",
		       mips_memory_upper);
	}
	mips_pci_io_base = MPU22X_PCIIO_WIN_BASE;
	mips_pci_io_size = MPU22X_PCIIO_WIN_SIZE;
	mips_pci_mem_base = MPU22X_PCIMEM_WIN0_BASE;
	mips_pci_mem_size = (MPU22X_PCIMEM_WIN0_SIZE +
			     MPU22X_PCIMEM_WIN1_SIZE +
			     MPU22X_PCIMEM_WIN2_SIZE);

	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_PCI2,
				       "0x%08lx=mips_pci_io_base\n",
				       mips_pci_io_base);
	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_PCI2,
				       "0x%08lx=mips_pci_io_size\n",
				       mips_pci_io_size);
	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_PCI2,
				       "0x%08lx=mips_pci_mem_base\n",
				       mips_pci_mem_base);
	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_PCI2,
				       "0x%08lx=mips_pci_mem_size\n",
				       mips_pci_mem_size);
	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_PCI2,
				       "0x%08lx=pci_io_resource.start\n",
				       pci_io_resource.start);
	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_PCI2,
				       "0x%08lx=pci_io_resource.end\n",
				       pci_io_resource.end);
	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_PCI2,
				       "0x%08lx=pci_mem_resource.start\n",
				       pci_mem_resource.start);
	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_PCI2,
				       "0x%08lx=pci_mem_resource.end\n",
				       pci_mem_resource.end);
	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_PCI2,
				       "0x%08lx=mips_io_port_base",
				       mips_io_port_base);

	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_PCI2,
				       "setup pci_io_resource  to 0x%08lx 0x%08lx\n",
				       pci_io_resource.start,
				       pci_io_resource.end);
	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_PCI2,
				       "setup pci_mem_resource to 0x%08lx 0x%08lx\n",
				       pci_mem_resource.start,
				       pci_mem_resource.end);

	if (!called) {
		printk
		    ("TX4927 PCIC -- DID:%04x VID:%04x RID:%02x Arbiter:%s\n",
		     (unsigned short) (tx4927_pcicptr->pciid >> 16),
		     (unsigned short) (tx4927_pcicptr->pciid & 0xffff),
		     (unsigned short) (tx4927_pcicptr->pciccrev & 0xff),
		     (!(tx4927_ccfgptr->
			ccfg & TX4927_CCFG_PCIXARB)) ? "External" :
		     "Internal");
		called = 1;
	}
	printk("TX4927 PCIC --%s PCICLK:",
	       (tx4927_ccfgptr->ccfg & TX4927_CCFG_PCI66) ? " PCI66" : "");
	if (tx4927_ccfgptr->pcfg & TX4927_PCFG_PCICLKEN_ALL) {
		int pciclk = 0;
                if (mpu22x_cputype == MPU22X_CPU_TX4927) {
                        switch ((unsigned long) tx4927_ccfgptr->
                                ccfg & TX4927_CCFG_PCIDIVMODE_MASK) {
                        case TX4927_CCFG_PCIDIVMODE_2_5:
                                pciclk = tx4927_cpu_clock * 2 / 5;
                                break;
                        case TX4927_CCFG_PCIDIVMODE_3:
                                pciclk = tx4927_cpu_clock / 3;
                                break;
                        case TX4927_CCFG_PCIDIVMODE_5:
                                pciclk = tx4927_cpu_clock / 5;
                                break;
                        case TX4927_CCFG_PCIDIVMODE_6:
                                pciclk = tx4927_cpu_clock / 6;
                                break;
                        }
                } else if (mpu22x_cputype == MPU22X_CPU_TX4937) {
                        /* XXX */
                        pciclk = tx4927_cpu_clock / 10;
                }
		printk("Internal(%dMHz)", pciclk / 1000000);
	} else {
		int pciclk = 0;
		int pciclk_setting = *tx4927_pci_clk_ptr;
		switch (pciclk_setting & TX4927_PCI_CLK_MASK) {
		case TX4927_PCI_CLK_33:
			pciclk = 33333333;
			break;
		case TX4927_PCI_CLK_25:
			pciclk = 25000000;
			break;
		case TX4927_PCI_CLK_66:
			pciclk = 66666666;
			break;
		case TX4927_PCI_CLK_50:
			pciclk = 50000000;
			break;
		}
		printk("External(%dMHz)", pciclk / 1000000);
	}
	printk("\n");



	/* GB->PCI mappings */
	tx4927_pcicptr->g2piomask = (mips_pci_io_size - 1) >> 4;
	tx4927_pcicptr->g2piogbase = mips_pci_io_base |
#ifdef __BIG_ENDIAN
	    TX4927_PCIC_G2PIOGBASE_ECHG
#else
	    TX4927_PCIC_G2PIOGBASE_BSDIS
#endif
	    ;

	tx4927_pcicptr->g2piopbase = 0;

#ifdef MPU22X_PCIMEM_WIN0_BASE
	tx4927_pcicptr->g2pmmask[0] = (MPU22X_PCIMEM_WIN0_SIZE - 1) >> 4;
	tx4927_pcicptr->g2pmgbase[0] = MPU22X_PCIMEM_WIN0_BASE |
#ifdef __BIG_ENDIAN
	    TX4927_PCIC_G2PMnGBASE_ECHG
#else
	    TX4927_PCIC_G2PMnGBASE_BSDIS
#endif
	    ;
	tx4927_pcicptr->g2pmpbase[0] = MPU22X_PCIMEM_WIN0_BASE;
#endif /* MPU22X_PCIMEM_WIN0_BASE */

#ifdef MPU22X_PCIMEM_WIN1_BASE
	tx4927_pcicptr->g2pmmask[1] = (MPU22X_PCIMEM_WIN1_SIZE - 1) >> 4;
	tx4927_pcicptr->g2pmgbase[1] = MPU22X_PCIMEM_WIN1_BASE |
#ifdef __BIG_ENDIAN
	    TX4927_PCIC_G2PMnGBASE_ECHG
#else
	    TX4927_PCIC_G2PMnGBASE_BSDIS
#endif
	    ;
	tx4927_pcicptr->g2pmpbase[1] = MPU22X_PCIMEM_WIN1_BASE;
#endif /* MPU22X_PCIMEM_WIN1_BASE */

#ifdef MPU22X_PCIMEM_WIN2_BASE
	tx4927_pcicptr->g2pmmask[2] = (MPU22X_PCIMEM_WIN2_SIZE - 1) >> 4;
	tx4927_pcicptr->g2pmgbase[2] = MPU22X_PCIMEM_WIN2_BASE |
#ifdef __BIG_ENDIAN
	    TX4927_PCIC_G2PMnGBASE_ECHG
#else
	    TX4927_PCIC_G2PMnGBASE_BSDIS
#endif
	    ;
	tx4927_pcicptr->g2pmpbase[2] = MPU22X_PCIMEM_WIN2_BASE;
#endif /* MPU22X_PCIMEM_WIN2_BASE */


	/* PCI->GB mappings (I/O 256B) */
	tx4927_pcicptr->p2giopbase = 0;	/* 256B */


#ifdef TX4927_SUPPORT_COMMAND_IO
	tx4927_pcicptr->p2giogbase = 0 | TX4927_PCIC_P2GIOGBASE_TIOEN |
#ifdef __BIG_ENDIAN
	    TX4927_PCIC_P2GIOGBASE_TECHG
#else
	    TX4927_PCIC_P2GIOGBASE_TBSDIS
#endif
	    ;
#else
	tx4927_pcicptr->p2giogbase = 0;
#endif


	/* PCI->GB mappings (MEM 512MB) */
	tx4927_pcicptr->p2gm0plbase = 0;
	tx4927_pcicptr->p2gmgbase[0] = 0 | TX4927_PCIC_P2GMnGBASE_TMEMEN |
#ifdef __BIG_ENDIAN
	    TX4927_PCIC_P2GMnGBASE_TECHG
#else
	    TX4927_PCIC_P2GMnGBASE_TBSDIS
#endif
	    ;


	/* PCI->GB mappings (none) */
	tx4927_pcicptr->p2gm1plbase = 0xffffffff;
	tx4927_pcicptr->p2gmgbase[1] = 0;

	tx4927_pcicptr->p2gm2pbase = 0xffffffff;
	tx4927_pcicptr->p2gmgbase[2] = 0;


	/* Enable Initiator Memory Spaces, I/O Space, Config */
	tx4927_pcicptr->pciccfg &= TX4927_PCIC_PCICCFG_LBWC_MASK;
	tx4927_pcicptr->pciccfg |=
#ifdef MPU22X_PCIMEM_WIN0_BASE
		TX4927_PCIC_PCICCFG_IMSE0 |
#endif
#ifdef MPU22X_PCIMEM_WIN1_BASE
		TX4927_PCIC_PCICCFG_IMSE1 |
#endif
#ifdef MPU22X_PCIMEM_WIN2_BASE
		TX4927_PCIC_PCICCFG_IMSE2 |
#endif
#ifdef MPU22X_PCIIO_WIN_BASE
		TX4927_PCIC_PCICCFG_IISE |
#endif
		TX4927_PCIC_PCICCFG_ICAE | TX4927_PCIC_PCICCFG_ATR;




	/* Do not use MEMMUL, MEMINF: YMFPCI card causes M_ABORT. */
	tx4927_pcicptr->pcicfg1 = 0;

	if (tx4927_pcic_retryto >= 0) {
		tx4927_pcicptr->g2ptocnt &= ~0xff00;
		tx4927_pcicptr->g2ptocnt |= (tx4927_pcic_retryto & 0xff00);
	}
	if (tx4927_pcic_trdyto >= 0) {
		tx4927_pcicptr->g2ptocnt &= ~0xff;
		tx4927_pcicptr->g2ptocnt |= (tx4927_pcic_trdyto & 0xff);
		//printk("TX4927 PCIC -- TRDYTO:%02lx\n",
		//      tx4927_pcicptr->g2ptocnt & 0xff);
	}

	/* Clear All Local Bus Status */
	tx4927_pcicptr->pcicstatus = TX4927_PCIC_PCICSTATUS_ALL;
	/* Enable All Local Bus Interrupts */
	tx4927_pcicptr->pcicmask = TX4927_PCIC_PCICSTATUS_ALL;
	/* Clear All Initiator Status */
	tx4927_pcicptr->g2pstatus = TX4927_PCIC_G2PSTATUS_ALL;
	/* Enable All Initiator Interrupts */
	tx4927_pcicptr->g2pmask = TX4927_PCIC_G2PSTATUS_ALL;
	/* Clear All PCI Status Error */
	tx4927_pcicptr->pcistatus =
	    (tx4927_pcicptr->pcistatus & 0x0000ffff) |
	    (TX4927_PCIC_PCISTATUS_ALL << 16);
	/* Enable All PCI Status Error Interrupts */
	tx4927_pcicptr->pcimask = TX4927_PCIC_PCISTATUS_ALL;

	/* PCIC Int => IRC IRQ16 */
	tx4927_pcicptr->pcicfg2 =
	    (tx4927_pcicptr->pcicfg2 & 0xffffff00) | TX4927_IR_PCIC;

	if (!(tx4927_ccfgptr->ccfg & TX4927_CCFG_PCIXARB)) {
		/* XXX */
	} else {
		/* Reset Bus Arbiter */
		tx4927_pcicptr->pbacfg = TX4927_PCIC_PBACFG_RPBA;
		/* Enable Bus Arbiter */
		tx4927_pcicptr->pbacfg = TX4927_PCIC_PBACFG_PBAEN;
	}

	tx4927_pcicptr->pcistatus = PCI_COMMAND_MASTER |
	    PCI_COMMAND_MEMORY |
#ifdef TX4927_SUPPORT_COMMAND_IO
	    PCI_COMMAND_IO |
#endif
	    PCI_COMMAND_PARITY | PCI_COMMAND_SERR;

	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_PCI2,
				       ":pci setup complete:\n");
//      tx4927_dump_pcic_settings();


	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_PCI2, "+\n");
	return;
}

#endif				/* CONFIG_PCI */


void snsc_mpu22x_restart(char *command)
{
	printk(KERN_NOTICE "System Rebooting...\n");

#ifdef CONFIG_SNSC_CXD9790
	cli();
	my_cxd9790_gpio_outbit(MPU22X_SELFRST_GPIO,
			       MPU22X_SELFRST_GPIO_BITNUM, 1);
	my_cxd9790_gpio_outbit(MPU22X_SELFRST_GPIO,
			       MPU22X_SELFRST_GPIO_BITNUM, 0);
#endif

	/* do something passive while waiting for reset */
	cli();
	while (1)
		asm_wait();

	/* no return */
}


void snsc_mpu22x_halt(void)
{
	printk(KERN_NOTICE "System Halted\n");
	cli();
	while (1) {
		asm_wait();
	}
	/* no return */
}


void snsc_mpu22x_power_off(void)
{
#ifdef CONFIG_SNSC_CXD9790
	cli();
	my_cxd9790_gpio_outbit(MPU22X_SHTDWNACK_GPIO,
			       MPU22X_SHTDWNACK_GPIO_BITNUM, 1);
	my_cxd9790_gpio_outbit(MPU22X_PWROFF_GPIO,
			       MPU22X_PWROFF_GPIO_BITNUM, 1);
	my_cxd9790_gpio_outbit(MPU22X_PWROFF_GPIO,
			       MPU22X_PWROFF_GPIO_BITNUM, 0);
#endif
	snsc_mpu22x_halt();
	/* no return */
}

#ifdef CONFIG_SNSC_HEARTBEAT_SIGNAL
static void snsc_mpu22x_do_heartbeat_signal(void)
{
	u32 reg_value;
	unsigned long flags;

	local_irq_save(flags);
	reg_value = tx4927_reg_read32(TX4927_PIO_PIOD0);
	reg_value ^= (1 << MPU22X_HEARTBEAT_LED_NUM);
	tx4927_reg_write32(TX4927_PIO_PIOD0, reg_value);
	local_irq_restore(flags);

	return;
}
#endif /* CONFIG_SNSC_HEARTBEAT_SIGNAL */

unsigned int __init
snsc_mpu22x_masterclk_freq(void)
{
	int cpuclk;

	cpuclk = ((TX4927_RD64(TX4927_BASE + TX4927_CONFIG_CCFG) >> 32) &
		  MPU22X_BOARDID_CPUCLK_MASK);
	switch (cpuclk) {
	case MPU22X_BOARDID_CPUCLK_30MHZ:
		return 30000000;	/* 30MHz */
	case MPU22X_BOARDID_CPUCLK_33MHZ:
		return 33000000;	/* 33MHz */
	}

	/* unknown board ID */
	printk(KERN_WARNING "Warning: unknown cpuclk 0x%02x!\n", cpuclk);
	return 33000000;		/* 33MHz */
}

void __init snsc_mpu22x_setup(void)
{
	vu32 cp0_config;

	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_SETUP,
				       "-\n");

	/* set TX4927 clocks' frequencies */
	{
		static const int divmode2times[16] = {
			16, 24, 32,  0,  4,  6,  8,  0,
			 0, 36,  0, 20,  0,  9,  0,  5
		};
		int divmode;

		tx4927_masterclk_freq = snsc_mpu22x_masterclk_freq();
		divmode = ((TX4927_RD64(TX4927_BASE + TX4927_CONFIG_CCFG) >>
			    17) & 0xf);
		tx4927_cpuclk_freq =
			tx4927_masterclk_freq * divmode2times[divmode] / 2;
		if (divmode & (1 << 2)) {
			tx4927_gbusclk_freq = tx4927_masterclk_freq;
		} else {
			tx4927_gbusclk_freq = tx4927_masterclk_freq * 4;
		}
	}

	/* read and save WDRST flag */
	tx4927_wdrst_flag = 0;
	if (TX4927_RD64(TX4927_BASE + TX4927_CONFIG_CCFG) & (1LL << 41)) {
		tx4927_wdrst_flag = 1;
	}

	/* f/w leaves this on at startup */
	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_SETUP,
				       ":Clearing STO_ERL.\n");
	clear_c0_status(ST0_ERL);

	/* enable caches -- HCP5 does this, pmon does not */
	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_SETUP,
				       ":Enabling TX49_CONF_IC,TX49_CONF_DC.\n");
	cp0_config = read_c0_config();
	cp0_config = cp0_config & ~(TX49_CONF_IC | TX49_CONF_DC);
	write_c0_config(cp0_config);

#ifdef SNSC_MPU22X_SETUP_DEBUG
	{
		extern void dump_cp0(char *);
		dump_cp0("snsc_mpu22x_early_fw_fixup");
	}
#endif

	/* setup irq stuff */
	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_SETUP,
				       ":Setting up tx4927 pic.\n");
	TX4927_WR(0xff1ff604, 0x00000000);	/* irq trigger */
	TX4927_WR(0xff1ff608, 0x00000000);	/* irq trigger */

	/* setup serial stuff */
	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_SETUP,
				       ":Setting up tx4927 sio.\n");
	TX4927_WR(0xff1ff314, 0x00000000);	/* h/w flow control off */
	TX4927_WR(0xff1ff414, 0x00000000);	/* h/w flow control off */

	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_SETUP,
				       "+\n");



	mips_io_port_base = KSEG1 + MPU22X_ISA_IO_OFFSET;
	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_SETUP,
				       ":mips_io_port_base=0x%08lx\n",
				       mips_io_port_base);

	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_SETUP,
				       ":Resource\n");
	ioport_resource.start = 0;
	ioport_resource.end = 0xffffffff;
	iomem_resource.start = 0;
	iomem_resource.end = 0xffffffff;


	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_SETUP,
				       ":ResetRoutines\n");
	_machine_restart = snsc_mpu22x_restart;
	_machine_halt = snsc_mpu22x_halt;
	_machine_power_off = snsc_mpu22x_power_off;


#ifdef CONFIG_BLK_DEV_IDEPCI
	{
		SNSC_MPU22X_SETUP_DPRINTK
		    (SNSC_MPU22X_SETUP_SETUP,
		     ":ide_ops=&std_ide_ops(modified)\n");
		ide_ops = &std_ide_ops;
	}
#else
	{
		SNSC_MPU22X_SETUP_DPRINTK
		    (SNSC_MPU22X_SETUP_SETUP,
		     ":ide_ops=<NOT_CONFIG>\n");
	}
#endif

#ifdef CONFIG_FB
	{
		conswitchp = &dummy_con;
	}
#endif




#ifdef CONFIG_PCI

	/* PCIC */
	/*
	   * ASSUMPTION: PCIDIVMODE is configured for PCI 33MHz or 66MHz.
	   * PCIDIVMODE[12:11]'s initial value are given by S9[4:3] (ON:0, OFF:1).
	   * CPU 166MHz: PCI 66MHz : PCIDIVMODE: 00 (1/2.5)
	   * CPU 200MHz: PCI 66MHz : PCIDIVMODE: 01 (1/3)
	   * CPU 166MHz: PCI 33MHz : PCIDIVMODE: 10 (1/5)
	   * CPU 200MHz: PCI 33MHz : PCIDIVMODE: 11 (1/6)
	   * i.e. S9[3]: ON (83MHz), OFF (100MHz)
	 */
	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_PCI1,
				       "ccfg is %lx, DIV is %x\n",
				       (unsigned long) tx4927_ccfgptr->
				       ccfg, TX4927_CCFG_PCIDIVMODE_MASK);

	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_PCI1,
				       "PCI66 mode is %lx, PCI mode is %lx, pci arb is %lx\n",
				       (unsigned long) tx4927_ccfgptr->
				       ccfg & TX4927_CCFG_PCI66,
				       (unsigned long) tx4927_ccfgptr->
				       ccfg & TX4927_CCFG_PCIMIDE,
				       (unsigned long) tx4927_ccfgptr->
				       ccfg & TX4927_CCFG_PCIXARB);

	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_PCI1,
				       "PCIDIVMODE is %lx\n",
				       (unsigned long) tx4927_ccfgptr->
				       ccfg & TX4927_CCFG_PCIDIVMODE_MASK);

         if (mpu22x_cputype == MPU22X_CPU_TX4927) {
                switch ((unsigned long) tx4927_ccfgptr->
                        ccfg & TX4927_CCFG_PCIDIVMODE_MASK) {
                case TX4927_CCFG_PCIDIVMODE_2_5:
                case TX4927_CCFG_PCIDIVMODE_5:
                        tx4927_cpu_clock = 166000000;	/* 166MHz */
                        break;
                default:
                        tx4927_cpu_clock = 200000000;	/* 200MHz */
                }
        } else if (mpu22x_cputype == MPU22X_CPU_TX4937) {
                /* XXX */
                tx4927_cpu_clock = tx4927_cpuclk_freq;
        }

	/* CCFG */
	/* enable Timeout BusError */
	if (tx4927_ccfg_toeon)
		tx4927_ccfgptr->ccfg |= TX4927_CCFG_TOE;

	/* SDRAMC fixup */
#ifdef CONFIG_TX4927BUG_WORKAROUND
	/*
	 * TX4927-BUG: INF 01-01-18/ BUG 01-01-22
	 * G-bus timeout error detection is incorrect
	 */
	if (tx4927_ccfg_toeon)
		tx4927_sdramcptr->tr |= 0x02000000;	/* RCD:3tck */
#endif

#ifdef  TX4927_SUPPORT_PCI_66
	tx4927_pci66_setup();
#endif

	tx4927_pci_setup();
#endif

	/* this is only done if backplane board installed, so must wait for pci */
#ifdef CONFIG_PC_KEYB
	{
		if (tx4927_using_backplane) {
			extern struct kbd_ops std_kbd_ops;
			kbd_ops = &std_kbd_ops;
			SNSC_MPU22X_SETUP_DPRINTK
			    (SNSC_MPU22X_SETUP_SETUP,
			     ":kbd_ops=&std_kbd_ops\n");
		} else {
			SNSC_MPU22X_SETUP_DPRINTK
			    (SNSC_MPU22X_SETUP_SETUP,
			     ":kbd_ops=<NO_BACKPLANE>\n");
		}
	}
#else
	{
		SNSC_MPU22X_SETUP_DPRINTK
		    (SNSC_MPU22X_SETUP_SETUP,
		     ":kbd_ops=<NOT_CONFIG>\n");
	}
#endif

	/* this is on ISA bus behind PCI bus, so need PCI up first */
#ifdef CONFIG_TOSHIBA_FPCIB0
	{
		if (tx4927_using_backplane) {
			SNSC_MPU22X_SETUP_DPRINTK
			    (SNSC_MPU22X_SETUP_SETUP,
			     ":fpcibo=yes\n");

			SNSC_MPU22X_SETUP_DPRINTK
			    (SNSC_MPU22X_SETUP_SETUP,
			     ":smsc_fdc37m81x_init()\n");
			smsc_fdc37m81x_init(0x3f0);

			SNSC_MPU22X_SETUP_DPRINTK
			    (SNSC_MPU22X_SETUP_SETUP,
			     ":smsc_fdc37m81x_config_beg()\n");
			smsc_fdc37m81x_config_beg();

			SNSC_MPU22X_SETUP_DPRINTK
			    (SNSC_MPU22X_SETUP_SETUP,
			     ":smsc_fdc37m81x_config_set(KBD)\n");
			smsc_fdc37m81x_config_set(SMSC_FDC37M81X_DNUM,
						  SMSC_FDC37M81X_KBD);
			smsc_fdc37m81x_config_set(SMSC_FDC37M81X_INT, 1);
			smsc_fdc37m81x_config_set(SMSC_FDC37M81X_INT2, 12);
			smsc_fdc37m81x_config_set(SMSC_FDC37M81X_ACTIVE,
						  1);

			smsc_fdc37m81x_config_end();
			SNSC_MPU22X_SETUP_DPRINTK
			    (SNSC_MPU22X_SETUP_SETUP,
			     ":smsc_fdc37m81x_config_end()\n");
		} else {
			SNSC_MPU22X_SETUP_DPRINTK
			    (SNSC_MPU22X_SETUP_SETUP,
			     ":fpcibo=not_found\n");
		}
	}
#else
	{
		SNSC_MPU22X_SETUP_DPRINTK
		    (SNSC_MPU22X_SETUP_SETUP, ":fpcibo=no\n");
	}
#endif


	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_SETUP,
				       "+\n");

	return;
}


void __init snsc_mpu22x_time_init(void)
{
	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_TIME_INIT,
				       "-\n");

	mips_counter_frequency = tx4927_cpuclk_freq / 2;

	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_TIME_INIT,
				       ":f1=%12u\n",
				       mips_counter_frequency);


	/* round to 1/10th of a MHz */
	mips_counter_frequency /= (100 * 1000);
	mips_counter_frequency *= (100 * 1000);
	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_TIME_INIT,
				       ":f2=%12u\n",
				       mips_counter_frequency);


	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_INFO,
				       ":mips_counter_frequency=%uHz (%uMHz)\n",
				       mips_counter_frequency,
				       mips_counter_frequency / 1000000);


	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_TIME_INIT,
				       "+\n");

#ifdef CONFIG_SNSC_MPU22X_SET_SYSCLOCK_BY_RTC
	{
		extern unsigned long rtc7301_rtc_rtc_get_time(void);
		extern unsigned long (*rtc_get_time)(void);

		rtc_get_time = rtc7301_rtc_rtc_get_time;
	}
#endif
#ifdef CONFIG_SNSC_HEARTBEAT_SIGNAL
	{
		extern void (*do_heartbeat_signal)(void);

		tx4927_reg_clearbits32(TX4927_PIO_PIOD0,
				       (1 << MPU22X_HEARTBEAT_LED_NUM));
		tx4927_reg_setbits32(TX4927_PIO_PIODIR,
				     (1 << MPU22X_HEARTBEAT_LED_NUM));
		
		do_heartbeat_signal = snsc_mpu22x_do_heartbeat_signal;
	}
#endif /* CONFIG_SNSC_HEARTBEAT_SIGNAL */

	return;
}


void __init snsc_mpu22x_timer_setup(struct irqaction *irq)
{
	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_TIMER_SETUP,
				       "-\n");
	SNSC_MPU22X_SETUP_DPRINTK(SNSC_MPU22X_SETUP_TIMER_SETUP,
				       "+\n");

	return;
}
