/* _FOR_OPTICAL_DISK_ */

/*
 * BRIEF MODULE DESCRIPTION
 *	The interface driver to access Sony XDCAM specific functions
 *
 * Copyright (C) 2003, 2004, 2005 Sony Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <linux/config.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/miscdevice.h>
#include <asm/uaccess.h>
#include <asm/io.h>
#include <asm/vr41xx.h>
#include <asm/vr4122/vr4122.h>
#include <asm/vr41xx/dif144.h>
#include <linux/dif144conf.h>

#undef DEBUG
#ifdef  DEBUG
#define DBG(x...) printk(x)
#else
#  define DBG(x...)
#endif

#define VR4122_PMUCNTREG KSEG1ADDR(0x0F0000C2)

static int dif144conf_open(struct inode *inode, struct file *file)
{
	DBG(__FUNCTION__ "\n");
	return 0;
}

static int dif144conf_close(struct inode *inode, struct file *file)
{
	DBG(__FUNCTION__ "\n");
	return 0;
}

void dif144conf_hibernate(void)
{
	__asm__ (
		".set\tnoreorder\n\t"
		".word\t0x42000023\n\t"
		"nop\n"
		"nop\n"
		"nop\n"
		"nop\n"
		"nop\n"
		".set reorder\n");
}

#define MTDBLOCK_MAJ 31

static void check_boot_mode(void)
{
	/* mtdblock? */
	if (MAJOR(ROOT_DEV) != MTDBLOCK_MAJ) return;

	/* recovery? */
	if (MINOR(ROOT_DEV) == 2 || MINOR(ROOT_DEV) == 9) return;

	/* itron is recovery? */
	if ((readw(VR4122_GIUPIODL) & 0x0050) == 0x0040) {
		printk("switching recovery mode\n");
		machine_restart();
	}
}

static int dif144conf_ioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg)
{
	DBG(__FUNCTION__ "\n");

	switch (cmd) {
		u16 val;
		u8 tp;
		u8 dev;
		case DIF144CONF_HALTIMER_CLEAR:
			val = readw(VR4122_PMUCNTREG);
			writew(val | 0x4, VR4122_PMUCNTREG);
			printk(KERN_INFO "dif144conf_ioctl: haltmer clear\n");
			break;
		case DIF144CONF_CHECK_BOOTMODE:
			check_boot_mode();
			break;
		case DIF144CONF_HIBERNATE:
			printk(KERN_INFO "dif144conf_ioctl: hibernate\n");
			dif144conf_hibernate();
			break;
		case DIF144CONF_TP_HI:
			get_user(tp, (unsigned char*)arg);
			dif144_tp_hi(tp);
			break;
		case DIF144CONF_TP_LOW:
			get_user(tp, (unsigned char*)arg);
			dif144_tp_low(tp);
			break;
		case DIF144CONF_DEV_ON:
			get_user(dev, (unsigned char*)arg);
			dif144_dev_clock(dev, 1);
			break;
		case DIF144CONF_DEV_OFF:
			get_user(dev, (unsigned char*)arg);
			dif144_dev_clock(dev, 0);
			break;
		default:
			printk(KERN_ERR "dif144conf_ioctl: unknown\n");
			return -ENOTTY;
	}

	return 0;
}

static struct file_operations dif144conf_fops = {
	owner: THIS_MODULE,
	open: dif144conf_open,
	ioctl: dif144conf_ioctl,
	release: dif144conf_close,
};

static struct miscdevice dif144conf = {
	243, "dif144conf", &dif144conf_fops
};


static int __init dif144conf_init_module(void)
{
	DBG(__FUNCTION__ "\n");

	return misc_register(&dif144conf);
}

static void __exit dif144conf_cleanup_module(void)
{
	DBG(__FUNCTION__ "\n");

	misc_deregister(&dif144conf);
}

module_init(dif144conf_init_module);
module_exit(dif144conf_cleanup_module);

MODULE_AUTHOR("Seiichi SATO");
MODULE_DESCRIPTION("Sony BSNC DIF-144/MPU-303 config interface driver");
MODULE_LICENSE("GPL");

