/*
 *  IDE driver for TDK XR1
 *
 *  Copyright 2003 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/types.h>
#include <linux/hdreg.h>
#include <linux/ide.h>
#include <linux/xr1.h>

#define XR1_IDEREG_OFFSET	0x200

/* ======================================================================= *
 * IDE driver interface                                                    *
 * ======================================================================= */

static int xr1_idereg_offsets[IDE_NR_PORTS] = {
	0, 2, 4, 6, 8, 10, 12, 14, 28, -1
};

void __init
xr1_init(void)
{
	hw_regs_t hw;
	ide_hwif_t *hwifp;
	int index;

	/* platform dependent initialization */
	xr1_platform_init();

	/* register myself as an IDE driver */
	ide_setup_ports(&hw, (ide_ioreg_t)(XR1_BASE + XR1_IDEREG_OFFSET),
			xr1_idereg_offsets, 0, 0, NULL, XR1_IRQ);

	index = ide_register_hw(&hw, &hwifp);
	if (index == -1) {
		return;
	}

#ifdef XR1_PLATFORM_IDEPROC
	/* platform dependent ideproc routine */
	hwifp->ideproc = xr1_platform_ideproc;
#endif
	printk("ide%d: TDK XR1 IDE interface\n", index);

	return;
}
