/*
 *  MPU-22X memory map 
 *
 *  Copyright 2003 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef _MPU220_MEMMAP_H_
#define _MPU220_MEMMAP_H_

/*
 *  SDRAM
 */
#define SDRAM_BASE                 0x00000000
#define SDRAM_SIZE                 0x04000000    /* 64MB */

/*
 *  PCI
 */

/* PCI memory windows and resource */
#define MPU22X_PCIMEM_WIN0_BASE    0x10000000
#define MPU22X_PCIMEM_WIN0_SIZE    0x08000000    /* 128 MB */
#define MPU22X_PCIMEM_WIN1_BASE    0x18000000
#define MPU22X_PCIMEM_WIN1_SIZE    0x04000000    /* 64 MB */
/* #define MPU22X_PCIMEM_WIN2_BASE */
#define MPU22X_PCIMEM_WIN2_SIZE    0x00000000

#define MPU22X_PCIMEM_RES_START    MPU22X_PCIMEM_WIN0_BASE
#define MPU22X_PCIMEM_RES_END      \
	(MPU22X_PCIMEM_RES_START + MPU22X_PCIMEM_WIN0_SIZE + \
	 MPU22X_PCIMEM_WIN1_SIZE + MPU22X_PCIMEM_WIN2_SIZE - 1)

/* PCI I/O window and resource */
#define MPU22X_PCIIO_WIN_BASE      0x1c000000
#define MPU22X_PCIIO_WIN_SIZE      0x01000000    /* 16 MB */
#define MPU22X_PCIIO_RES_START     MPU22X_PCIIO_WIN_BASE
#define MPU22X_PCIIO_RES_END       \
	(MPU22X_PCIIO_WIN_BASE + MPU22X_PCIIO_WIN_SIZE - 1)

/* 
 *  CXD9790(/CE2)
 */
#define CXD9790_BASE               0x1d000000

/* CXD9790 GPIO */
#define     GPIO_PWROFF            ((CXD9790_GPIOZ << 8) | 0x00)
#define     GPIO_PWRGOOD           ((CXD9790_GPIOZ << 8) | 0x01)
#define     GPIO_SELFRST           ((CXD9790_GPIOZ << 8) | 0x02)
#define     GPIO_SHTDWNREQ         ((CXD9790_GPIOZ << 8) | 0x03)
#define     GPIO_SHTDWNACK         ((CXD9790_GPIOZ << 8) | 0x04)
#define     GPIO_STANDBY           ((CXD9790_GPIOZ << 8) | 0x05)
#define     GPIO_RECOVERY          ((CXD9790_GPIOZ << 8) | 0x06)
#define     GPIO_SRPCIALL          ((CXD9790_GPIOZ << 8) | 0x07)

#define     GPIO_PCPCI2            ((CXD9790_GPIOA << 8) | 0x00)
#define     GPIO_PCJIG             ((CXD9790_GPIOA << 8) | 0x01)
#define     GPIO_SRPCI2            ((CXD9790_GPIOA << 8) | 0x02)
#define     GPIO_SRJIG             ((CXD9790_GPIOA << 8) | 0x03)
#define     GPIO_INTJIG0           ((CXD9790_GPIOA << 8) | 0x04)
#define     GPIO_INTJIG1           ((CXD9790_GPIOA << 8) | 0x05)
#define     GPIO_INTRTC            ((CXD9790_GPIOA << 8) | 0x06)
#define     GPIO_JIGINS            ((CXD9790_GPIOA << 8) | 0x07)

#define     GPIO_GPIOJIG0          ((CXD9790_GPIOB << 8) | 0x00)
#define     GPIO_GPIOJIG1          ((CXD9790_GPIOB << 8) | 0x01)
#define     GPIO_SRPCI3            ((CXD9790_GPIOB << 8) | 0x02)
#define     GPIO_PCIPWROFF         ((CXD9790_GPIOB << 8) | 0x03)
#define     GPIO_PCPCI3            ((CXD9790_GPIOB << 8) | 0x04)
#define     GPIO_NORSEL0           ((CXD9790_GPIOB << 8) | 0x06)
#define     GPIO_NORSEL1           ((CXD9790_GPIOB << 8) | 0x07)

#define     GPREG(x,o)             (((x) >> 8) + (o))
#define     GPMSK(x)               (1 << ((x) & 0xff))


/* 
 *  NAND(/CE3)
 */
#define NAND_BASE                  0x1d100000


/* 
 *  RTC(/CE4)
 */
#define RTC_BASE                   0x1d200000


/*
 * NOR 
 */
#define NOR0_BASE_32               0x1f000000
#define NOR0_BASE_16               0x1f800000
#define NOR0_BASE_8                0x1fc00000
#define NOR1_BASE                  0x1fe00000


/*
 * misc definitions
 */

/* CPU-PIO */
#define CPIO_LED_MASK              (0x07e3)
#define CPIO_NAND_CS1              (11)
#define CPIO_NAND_CS0              (12)
#define CPIO_FLASH_BY              (13)
#define CPIO_NAND_CLE              (14)
#define CPIO_NAND_ALE              (15)

#endif /* _MPU220_MEMMAP_H_ */
