/*
 * linux/include/asm-mips/tx4927/snsc_mpu22x/snsc_mpu22x.h
 *
 * Sony NSC MPU-22X board definitions
 *
 * Copyright 2003 Sony Corporation.
 *
 * This file is based on linux/include/asm-mips/tx4927/toshiba_rbtx4927/toshiba_rbtx4927.h.
 *
 * linux/include/asm-mips/tx4927/toshiba_rbtx4927/toshiba_rbtx4927.h
 *
 * rbtx4927 defs
 *
 * Author: MontaVista Software, Inc.
 *         source@mvista.com
 *
 * Copyright 2001-2002 MontaVista Software Inc.
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 *  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 *  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef _TX4927_SNSC_MPU22X_
#define _TX4927_SNSC_MPU22X_

#include <linux/config.h>
#include <linux/cxd9790_reg.h>
#include <asm/tx4927/tx4927.h>
#include <asm/tx4927/snsc_mpu22x/mpu22x-memmap.h>

/*
 *  IRQs
 */
/* PCI interrupts */
#define MPU22X_IRQ_PCIA            (TX4927_IRQ_PIC_BEG + 2)
#define MPU22X_IRQ_PCIB            (TX4927_IRQ_PIC_BEG + 3)
#define MPU22X_IRQ_PCIC            (TX4927_IRQ_PIC_BEG + 4)
#define MPU22X_IRQ_PCID            (TX4927_IRQ_PIC_BEG + 5)

/* CXD9790 */
#define MPU22X_IRQ_CXD9790_OUT0    (TX4927_IRQ_PIC_BEG + 6)
#define MPU22X_IRQ_CXD9790_OUT1    (TX4927_IRQ_PIC_BEG + 7)

/* CPU UART */
#define MPU22X_IRQ_SIO0            (TX4927_IRQ_PIC_BEG + 8)
#define MPU22X_IRQ_SIO1            (TX4927_IRQ_PIC_BEG + 9)

int snsc_mpu22x_irq_nested0(int sw_irq);
int snsc_mpu22x_irq_nested1(int sw_irq);


/*
 *  PCI
 */
#ifdef CONFIG_PCI
#define MPU22X_ISA_IO_OFFSET       MPU22X_PCIIO_WIN_BASE
#else
#define MPU22X_ISA_IO_OFFSET 0
#endif /* CONFIG_PCI */


/*
 * CPU type
 */
#define MPU22X_CPU_TX_UNKNOWN             (0x0)
#define MPU22X_CPU_TX4927                 (0x1)
#define MPU22X_CPU_TX4937                 (0x2)
extern int mpu22x_cputype;

/* Board ID */
#define MPU22X_BOARDID_CPUCLK_MASK	(0x3 << 6)
#define  MPU22X_BOARDID_CPUCLK_33MHZ	(0 << 6)
#define  MPU22X_BOARDID_CPUCLK_30MHZ	(1 << 6)
#define MPU22X_BOARDID_SDRAM_MASK	(0x3 << 4)
#define  MPU22X_BOARDID_SDRAM_128MBIT	(0 << 4)
#define  MPU22X_BOARDID_SDRAM_256MBIT	(1 << 4)
#define  MPU22X_BOARDID_SDRAM_512MBIT	(2 << 4)
#define MPU22X_BOARDID_BOARDID_MASK	(0xf << 0)
#define  MPU22X_BOARDID_BOARDID_MPU220	(0 << 0)
#define  MPU22X_BOARDID_BOARDID_MPU221	(1 << 0)

/* LED for heartbeat signal */
#define MPU22X_HEARTBEAT_LED_NUM	7

/* Boot time analysis data */
#define BOOT_TIME_SIZE	0x1000	/* 4KB */
#define BOOT_TIME_BASE	(0x000ff000 - BOOT_TIME_SIZE)

/* NBLArgs */
#define NBLARGS_STD_ADDR        0x800ff000
#define NBLARGS_STD_LEN         0x1000

/* constant loops_per_jiffy */
#ifndef __ASSEMBLY__
#ifdef CONFIG_SNSC_CONST_LPJ
static inline unsigned long
lookup_lpj(void)
{
	switch (tx4927_cpuclk_freq) {
	case 300000000:	/* 300MHz */
		return 1499136;
	case 330000000:	/* 330MHz */
		return 1646592;
	}
	return 0;
}
#endif /* CONFIG_SNSC_CONST_LPJ */
#endif /* !__ASSEMBLY__ */

#endif /* _TX4927_SNSC_MPU22X_ */
