/*
 *  IDE driver for TDK XR1  (MPU-22x)
 *
 *  Copyright 2003 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef MPU22X_XR1_H_
#define MPU22X_XR1_H_

#include <linux/cxd9790.h>
#include <linux/cxd9790_gpio.h>
#include <asm-mips/io.h>
#include <asm-mips/tx4927/snsc_mpu22x/mpu22x-memmap.h>
#define XR1_BASE (KSEG1ADDR(NAND_BASE) - mips_io_port_base)
#define XR1_INTR_GPIO_PORT	GPIO_PORT_B
#define XR1_INTR_GPIO_BITNUM	6
#define XR1_IRQ  CXD9790_IRQ_GPIOB(XR1_INTR_GPIO_BITNUM)
#define XR1_DEVNAME	"XR1 IDE"

static inline void
xr1_platform_init(void)
{
	/* GPIO port setting */

	/* NOTE: no error check */
	cxd9790_gpio_init();
	cxd9790_gpio_register_bit(XR1_INTR_GPIO_PORT, XR1_INTR_GPIO_BITNUM,
				  XR1_DEVNAME);
	cxd9790_gpio_set_mode_bit(XR1_INTR_GPIO_PORT, XR1_INTR_GPIO_BITNUM,
				  GPIO_MODE_IN);
	cxd9790_gpio_intr_set_mode(XR1_INTR_GPIO_PORT, XR1_INTR_GPIO_BITNUM,
				   GPIO_INTR_LEVEL_LOW);

	return;
}

/* platform dependent ideproc routine is supported for burst access to XR1 */
#define XR1_PLATFORM_IDEPROC

#define DCACHE_LINE_WORD    0x8

static void 
xr1_platform_ideproc(ide_ide_action_t action, ide_drive_t *drive,
		     void *buffer, unsigned int count)
{
	int alsize;
	spinlock_t lock = SPIN_LOCK_UNLOCKED;
	unsigned long flags;

	switch(action) {
	case ideproc_ide_input_data:
		/* length aligned by cache line size */
		alsize = (count / DCACHE_LINE_WORD) * DCACHE_LINE_WORD;

		spin_lock_irqsave(&lock, flags);
		__asm__ __volatile__ (
			".set  push             \n"
			".set  mips3            \n"
			"1:                     \n"
			"    ld     $11, 0(%1)  \n"
			"    ld     $12, 8(%1)  \n"
			"    ld     $13, 16(%1) \n"
			"    ld     $14, 24(%1) \n"
			"    addiu  %2, %2, 32  \n"
			"    sd     $11, -32(%2)\n"
			"    sd     $12, -24(%2)\n"
			"    sd     $13, -16(%2)\n"
			"    sd     $14, -8(%2) \n"
			"    cache  %0,  (%1)   \n"
			"    bne    %2, %3, 1b  \n"
			"    nop                \n"
			".set  pop                "
			:
			: "i" (0x11), /* Hit Invalidate */
			"r" (KSEG0ADDR(NAND_BASE)),
			"r" (buffer),
			"r" ((u_int32_t *)buffer + alsize)
			);
		spin_unlock_irqrestore(&lock, flags);
		count -= alsize;
		if (count > 0) {
			buffer = (void *)((u_int32_t *)buffer + alsize);
			insw(IDE_DATA_REG, buffer, count<<1);
		}
		break;
	case ideproc_ide_output_data:
		/* length aligned by cache line size */
		alsize = (count / DCACHE_LINE_WORD) * DCACHE_LINE_WORD;

		spin_lock_irqsave(&lock, flags);
		__asm__ __volatile__ (
			".set  push             \n"
			".set  mips3            \n"
			"1:                     \n"
			"    ld     $11, 0(%2)  \n"
			"    ld     $12, 8(%2)  \n"
			"    ld     $13, 16(%2) \n"
			"    ld     $14, 24(%2) \n"
			"    addiu  %2, %2, 32  \n"
			"    sd     $11, 0(%1)  \n"
			"    sd     $12, 8(%1)  \n"
			"    sd     $13, 16(%1) \n"
			"    sd     $14, 24(%1) \n"
			"    cache  %0,  (%1)   \n"
			"    bne    %2, %3, 1b  \n"
			"    nop                \n"
			".set  pop                "
			:
			: "i" (0x15), /* Hit Writeback Invalidate */
			"r" (KSEG0ADDR(NAND_BASE)),
			"r" (buffer),
			"r" ((u_int32_t *)buffer + alsize)
			);
		spin_unlock_irqrestore(&lock, flags);
		count -= alsize;
		if (count > 0) {
			buffer = (void *)((u_int32_t *)buffer + alsize);
			outsw(IDE_DATA_REG, buffer, count<<1);
		}
		break;
	case ideproc_atapi_input_bytes:
		count++;
		insw(IDE_DATA_REG, buffer, count>>1);
		if ((count & 0x03) >= 2)
			insw (IDE_DATA_REG, ((byte *)buffer) + (count & ~0x03), 1);
		break;
	case ideproc_atapi_output_bytes:
		count++;
		outsw(IDE_DATA_REG, buffer, count>>1);
		if ((count & 0x03) >= 2)
			outsw (IDE_DATA_REG, ((byte *)buffer) + (count & ~0x03), 1);
		break;
	}
}

#endif /* !MPU22X_XR1_H_ */
