/* _FOR_OPTICAL_DISK_ */
/*
 *
 *  BRIEF MODULE DESCRIPTION
 *	Include file for Sony BSNC DIF-144 board.
 *
 *  Copyright 2002, 2003, 2004, 2005 Sony Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#ifndef __ASM_DIF144_H
#define __ASM_DIF144_H

#ifdef __KERNEL__

#include <asm/addrspace.h>
#include <asm/vr41xx/vr41xx.h>

void dif144_led_light(u8 mask, u8 mode);
u8 dif144_get_lighted_led(void);
void dif144_dev_clock(u8 dev, int on);
void dif144_tp_hi(u8 port);
void dif144_tp_low(u8 port);
u8 dif144_dipswitch(u16 mask);
int dif144_boot_mode(void);
void dif144_invalid_cache(void);

/*
 * PCIU address mappings
 */
#define VR41XX_PCI_MEM1_BASE		0x10000000
#define VR41XX_PCI_MEM1_SIZE		0x04000000
#define VR41XX_PCI_MEM1_MASK		0x7c000000

#define VR41XX_PCI_MEM2_BASE		0x14000000
#define VR41XX_PCI_MEM2_SIZE		0x02000000
#define VR41XX_PCI_MEM2_MASK		0x7e000000

#define VR41XX_PCI_IO_BASE		0x16000000
#define VR41XX_PCI_IO_SIZE		0x02000000
#define VR41XX_PCI_IO_MASK		0x7e000000

#define VR41XX_PCI_IO_START		0x00000000
#define VR41XX_PCI_IO_END		0x01ffffff

#define VR41XX_PCI_MEM_START		0x12000000
#define VR41XX_PCI_MEM_END		0x15ffffff

#define IO_PORT_BASE			KSEG1ADDR(VR41XX_PCI_IO_BASE)
#define IO_PORT_RESOURCE_START		0
#define IO_PORT_RESOURCE_END		VR41XX_PCI_IO_SIZE
#define IO_MEM1_RESOURCE_START		VR41XX_PCI_MEM1_BASE
#define IO_MEM1_RESOURCE_END		(VR41XX_PCI_MEM1_BASE + VR41XX_PCI_MEM1_SIZE)
#define IO_MEM2_RESOURCE_START		VR41XX_PCI_MEM2_BASE
#define IO_MEM2_RESOURCE_END		(VR41XX_PCI_MEM2_BASE + VR41XX_PCI_MEM2_SIZE)

/*
 * IRQs
 */
#define BSNC_DIF144_IRQ_PUSHSW1		GIU_IRQ(2)
#define BSNC_DIF144_IRQ_PUSHSW2		GIU_IRQ(3)
#define BSNC_DIF144_IRQ_ITRON		GIU_IRQ(4)
#define BSNC_DIF144_IRQ_INTEL_21555	GIU_IRQ(5)
#define BSNC_DIF144_IRQ_TI_PCI1510	GIU_IRQ(8)
#define BSNC_DIF144_IRQ_FPGA		GIU_IRQ(9)
#define BSNC_DIF144_IRQ_OSD         	GIU_IRQ(13)
#define BSNC_DPR238_IRQ_ETHER	    	GIU_IRQ(8)

#endif /* __KERNEL__ */

/*
 * External I/O space
 */
#define BSNC_DIF144_IO			KSEG1ADDR(0x0A000000)
#define BSNC_DIF144_DIPSW		BSNC_DIF144_IO
#define BSNC_DIF144_LED			BSNC_DIF144_IO

#define BSNC_DIF144_IO_DEFAULT		0x00ff 	/* TP310-308: LOW, LED*: off */

#define BSNC_DIF144_LED_MODE_OVERWRITE	0
#define BSNC_DIF144_LED_MODE_OR		1

#define BSNC_DIF144_LED_BIT0		1 
#define BSNC_DIF144_LED_BIT1		(1 << 1)
#ifdef CONFIG_MIPS_BSNC_DPR238
#  define BSNC_DIF144_LED_BIT2		(1 << 2)
#  define BSNC_DIF144_LED_BIT3		(1 << 3)
#  define BSNC_DIF144_LED_BIT4		(1 << 4)
#  define BSNC_DIF144_LED_BIT5		(1 << 5)
#  define BSNC_DIF144_LED_BIT6		(1 << 6)
#  define BSNC_DIF144_LED_BIT7		(1 << 7)
#else
#  define BSNC_DIF144_PCCARD_SHT_DWN_X	(1 << 2)
#  define BSNC_DIF144_OUT_OE_X		(1 << 3)
#  define BSNC_DIF144_PCCARD_DWN_X	(1 << 4)
#  define BSNC_DIF144_OSD_DWN_X		(1 << 5)
#  define BSNC_DIF144_BRIDGE_DWN_X	(1 << 6)
#  define BSNC_DIF144_FLS_PRG		(1 << 7)
#endif

#define BSNC_DIF144_TP301		7 
#define BSNC_DIF144_TP302		6 
#define BSNC_DIF144_TP303		5 
#define BSNC_DIF144_TP304		4 
#define BSNC_DIF144_TP305		3 
#define BSNC_DIF144_TP306		2 
#define BSNC_DIF144_TP307		1 
#define BSNC_DIF144_TP308		0 


#define BSNC_DIF144_DIPSW_S301_1        (1 << 15)
#define BSNC_DIF144_DIPSW_S301_2        (1 << 14)
#define BSNC_DIF144_DIPSW_S301_3        (1 << 13)
#define BSNC_DIF144_DIPSW_S301_4        (1 << 12)
#define BSNC_DIF144_DIPSW_S301_5        (1 << 11)
#define BSNC_DIF144_DIPSW_S301_6        (1 << 10)
#define BSNC_DIF144_DIPSW_S301_7        (1 << 9)
#define BSNC_DIF144_DIPSW_S301_8        (1 << 8)
#define BSNC_DIF144_DIPSW_S302_1        (1 << 7)
#define BSNC_DIF144_DIPSW_S302_2        (1 << 6)
#define BSNC_DIF144_DIPSW_S302_3        (1 << 5)
#define BSNC_DIF144_DIPSW_S302_4        (1 << 4)
#define BSNC_DIF144_DIPSW_S302_5        (1 << 3)
#define BSNC_DIF144_DIPSW_S302_6        (1 << 2)
#define BSNC_DIF144_DIPSW_S302_7        (1 << 1)
#define BSNC_DIF144_DIPSW_S302_8        1

#define BSNC_DIF144_DIPSW_JIG_BOOT_OLD  BSNC_DIF144_DIPSW_S302_4
#define BSNC_DIF144_DIPSW_JIG_BOOT      BSNC_DIF144_DIPSW_S302_7
#define BSNC_DIF144_DIPSW_SEL_ROOTFS    BSNC_DIF144_DIPSW_S302_5
#define BSNC_DIF144_DIPSW_SEL_KERNEL    BSNC_DIF144_DIPSW_S302_6
#define BSNC_DIF144_DIPSW_SEL_BY_DIPSW  BSNC_DIF144_DIPSW_S302_2
#define BSNC_DIF144_DIPSW_PPB_CONNECTED BSNC_DIF144_DIPSW_S302_1

//#define BSNC_DIF144_PIN_FAN_LOCK        BSNC_DIF144_DIPSW_S302_7
#define BSNC_DIF144_PIN_JIG_EXIST       BSNC_DIF144_DIPSW_S302_8

#define BSNC_DIF144_DIPSW_ON            1
#define BSNC_DIF144_DIPSW_OFF           0

#define BSNC_DIF144_MODE_A		0
#define BSNC_DIF144_MODE_B		1


#endif /* __ASM_DIF144_H */
